package Refprop9;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(Load_REFPROP Get_Fluid_Property);
@EXPORT_OK = qw(@AVAIL_REF_FUNCS @AVAIL_PROPERTIES @REF_DATA_LOCS);

use strict;
use Win32;
use Win32::API 0.68;
use Cwd qw (cwd abs_path);
use File::Basename qw (fileparse);
use Scalar::Util qw (looks_like_number);

my $Loading_Warning = "Load RefProp.DLL First";

#PUBLIC VARIABLES##########################
our (@AVAIL_REF_FUNCS,@AVAIL_PROPERTIES,@REF_DATA_LOCS);
@AVAIL_REF_FUNCS = ($Loading_Warning);
@REF_DATA_LOCS = ($Loading_Warning);

#Data locations will be set
my ($RefPropDLL,$FluidsDirectory,$MixturesDirectory);	#Variables will be used for paths to required data

#Setup of global functions in order to call from DLL
my ($ABFL1dll,$ABFL2dll,$ACTVYdll,$AGdll,$CCRITdll,$CHEMPOTdll,$CRITPdll,$CSATKdll,$CSTARdll,$CV2PKdll,$CVCPdll);
my ($DBDTdll,$DBFL2dll,$DDDPdll,$DDDTdll,$DEFLSHdll,$DHD1dll,$DHFLSHdll,$DIELECdll,$DPDD2dll,$DPDDdll,$DPDTdll);
my ($DPTSATKdll,$DSFLSHdll,$ENTHALdll,$ENTROdll,$ESFLSHdll,$ETAK0dll,$ETAK1dll,$ETAKBdll,$ETAKRdll,$FGCTYdll,$FPVdll);
my ($FUGCOFdll,$GERG04dll,$GETFIJdll,$GETKTVdll,$HEATdll,$HSFLSHdll,$INFOdll,$LIMITKdll,$LIMITXdll,$MAXPdll,$MAXTdll);	
my ($MELTPdll,$MELTTdll,$MLTH2Odll,$OMEGAdll,$PDFLSHdll,$PEFLSHdll,$PHFLSHdll,$PQFLSHdll,$PREOSdll,$PRESSdll,$PSFLSHdll);
my ($PUREFLDdll,$QMASSdll,$QMOLEdll,$SATDATAdll,$SATDdll,$SATEdll,$SATHdll,$SATPdll,$SATSdll,$SATTdll,$SATTPdll,$SETAGAdll);
my ($SETKTVdll,$SETMIXdll,$SETMODdll,$SETREFdll,$SETUPdll,$SPECGRdll,$SUBLPdll,$SUBLTdll,$SURFTdll,$SURTENdll,$TCXK0dll);
my ($TCXKBdll,$TCXKCdll,$TDFLSHdll,$TEFLSHdll,$THERM0dll,$THERM2dll,$THERM3dll,$THERMdll,$THFLSHdll,$TPFLSHdll,$TPRHOdll);
my ($TQFLSHdll,$TRNPRPdll,$TSFLSHdll,$UNSETAGAdll,$VIRBdll,$VIRCdll,$WMOLdll,$XMASSdll,$XMOLEdll);

#CONSTANTS
my $MaxComps = 20;
my $Rgas = 8.314472;	#VBA As Double

#....Conversions
my $CtoK = 273.15;              	#Exact conversion
my $FtoR = 459.67;              	#Exact conversion
my $RtoK = 5 / 9;               	#Exact conversion
my $HtoS = 3600;                	#Exact conversion
my $ATMtoMPa = 0.101325;        	#Exact conversion
my $BARtoMPA = 0.1;             	#Exact conversion
my $KGFtoN = 98.0665 / 10;      	#Exact conversion
my $INtoM = 0.0254;             	#Exact conversion
my $FTtoM = 12 * $INtoM;        	#Exact conversion
my $LBMtoKG = 0.45359237;       	#Exact conversion
my $CALtoJ = 4.184;             	#Exact conversion (tc)
#my $CALtoJ = 4.1868;           	#Exact conversion (IT) (Use this one only with pure water)
my $MMHGtoMPA = $ATMtoMPa / 760;	#Exact conversion
my $INH2OtoMPA = 0.000249082;
my $BTUtoKJ = $CALtoJ * $LBMtoKG * $RtoK;
my $LBFtoN = $LBMtoKG * $KGFtoN;
my $IN3toM3 = $INtoM * $INtoM * $INtoM;
my $FT3toM3 = $FTtoM * $FTtoM * $FTtoM;
my $GALLONtoM3 = $IN3toM3 * 231;
my $PSIAtoMPA = $LBMtoKG / $INtoM / $INtoM * $KGFtoN / 1000000;
my $FTLBFtoJ = $FTtoM * $LBFtoN;
my $HPtoW = 550 * $FTLBFtoJ;
my $BTUtoW = $BTUtoKJ * 1000;
my $LBFTtoNM = $LBFtoN / $FTtoM;

#VARIABLES
my $CompFlag;	#VBA As Integer

#MODULE LEVEL VARIABLES USED TO CALL REFPROP
my ($herr, $herr2, $hfmix, $hfmix2, $hrf, $htyp, $hmxnme, $hfld, $hfldOld) = ( " " x 255,  " " x 255,  " " x 255,  " " x 255,  " " x 3,  " " x 3,  " " x 255,  " " x 10000,  " " x 10000);	#VBA As String (fixed)
my ($nc, $phase, $molmass, $ierr, $ierr2, $kq, $kr, $nroot, $k1, $k2, $k3);	#VBA As Long
my (@x, @xliq, @xvap, @xmm, @xkg, @xmol, @xdew, @ybub);		#VBA (1 To MaxComps) As Double
my ($wmix, $t, $p, $d, $Dl, $Dv, $q, $wm, $tz, $pz, $dz, $dd, $tbub, $tdew, $pbub, $pdew, $Dlbub, $Dvdew);	#VBA As Double
my ($e, $h, $s, $Cvcalc, $Cpcalc, $w, $tmin, $tmax, $dmax, $pmax, $tc, $pc, $dc, $eta, $tcx, $sigma, $hjt);	#VBA As Double
my ($de, $wmm, $ttrp, $tnbpt, $Zc, $acf, $dip, $z, $aHelm, $Gibbs, $xkappa, $beta, $dPdD_T, $epsk, $tcx0);	#VBA As Double
my ($d2PdD2_rho, $dPdT_rho, $dDdT_P, $dDdP_rho, $d2PT2, $d2PdTD, $spare3, $spare4, $tcxb, $tcxc, $omg);		#VBA As Double
my ($isenk, $kt, $betas, $bs, $kkt, $thrott, $xpi, $spht, $t2, $p2, $d2, $t3, $p3, $d3, $eta0, $eta1, $etar, $etab);	#VBA As Double
my (@tUnits, @taUnits, @pUnits, @dUnits, @vUnits, @hUnits, @sUnits, @wUnits, @visUnits, @tcxUnits, @stUnits);	#VBA (10) As String
my ($tUnits2, $taUnits2, $pUnits2, $dUnits2, $vUnits2, $hUnits2, $sUnits2, $wUnits2, $visUnits2, $tcxUnits2, $stUnits2, $FldOld);	#VBA As String

#OTHER MODULE VARIABLES
my (%triggers);
my $Func_Temperature = sub{	#(FluidName, InpCode, Optional Units, Optional Prop1, Optional Prop2)
	my ($FluidName,$InpCode,$Units,$Input1,$Input2) = @_;
	&CalcProp($FluidName, $InpCode, $Units, $Input1, $Input2);
	return &ConvertUnits("-T", $Units, $t, 0);
};
my $Func_Pressure = sub{	#(FluidName, InpCode, Optional Units, Optional Prop1, Optional Prop2)
	my ($FluidName,$InpCode,$Units,$Input1,$Input2) = @_;
	&CalcProp($FluidName, $InpCode, $Units, $Input1, $Input2);
	return &ConvertUnits("-P", $Units, $p, 0);
};
my $Func_Density = sub{	#(FluidName, InpCode, Optional Units, Optional Prop1, Optional Prop2)
	my ($FluidName,$InpCode,$Units,$Input1,$Input2) = @_;
	&CalcProp($FluidName, $InpCode, $Units, $Input1, $Input2);
	return &ConvertUnits("-D", $Units, $d, 0);
};
my $Func_Cp = sub{	#(FluidName, InpCode, Optional Units, Optional Prop1, Optional Prop2)
	my ($FluidName,$InpCode,$Units,$Input1,$Input2) = @_;
	&CalcProp($FluidName, $InpCode, $Units, $Input1, $Input2);
	return &ConvertUnits("-S", $Units, $Cpcalc, 0);
};
my $Func_Enthalpy = sub{	#(FluidName, InpCode, Optional Units, Optional Prop1, Optional Prop2)
	my ($FluidName,$InpCode,$Units,$Input1,$Input2) = @_;
	&CalcProp($FluidName, $InpCode, $Units, $Input1, $Input2);
	return &ConvertUnits("-H", $Units, $h, 0);
};
my $Func_Entropy = sub{	#(FluidName, InpCode, Optional Units, Optional Prop1, Optional Prop2)
	my ($FluidName,$InpCode,$Units,$Input1,$Input2) = @_;
	&CalcProp($FluidName, $InpCode, $Units, $Input1, $Input2);
	return &ConvertUnits("-S", $Units, $s, 0);
};
my $Func_Volume = sub{	#(FluidName, InpCode, Optional Units, Optional Prop1, Optional Prop2)
	my ($FluidName,$InpCode,$Units,$Input1,$Input2) = @_;
	my $v;
	&CalcProp($FluidName, $InpCode, $Units, $Input1, $Input2);
	return &Trim_Then_Append("Density is zero","#") if ($d <= 0);
	$v = 1 / $d;
	return &ConvertUnits("-V", $Units, $v, 0);
};
my $Func_Viscosity = sub{	#(FluidName, InpCode, Optional Units, Optional Prop1, Optional Prop2)
	my ($FluidName,$InpCode,$Units,$Input1,$Input2) = @_;
	my $viscosity_value;
	our $TRNPRPdll;
	&CalcProp($FluidName, $InpCode, $Units, $Input1, $Input2);
	return &Trim_Then_Append($herr,"#") if ($ierr > 0);
	return &Trim_Then_Append("Inputs out of range","#") if (($t == 0) || ($d == 0));
	($t, $d, $x[1], $eta, $tcx, $ierr2, $herr2) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$eta), pack('d',$tcx), pack('L!',$ierr2), pack('A255',$herr2));
	$TRNPRPdll->Call($t, $d, $x[1], $eta, $tcx, $ierr2, $herr2, pack('L!',255));
	($t, $d, $x[1], $eta, $tcx, $ierr2, $herr2) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$eta), unpack('d',$tcx), unpack('L!',$ierr2), unpack('A255',$herr2));
	$eta = -9999999 if (($q > 0) && ($q < 1));
	$viscosity_value = &ConvertUnits("-U", $Units, $eta, 0);
	unless($eta){
		return &Trim_Then_Append("Unable to calculate property","#");
	}else{return $viscosity_value;}
};
my $Func_Quality = sub{	#(FluidName, InpCode, Optional Units, Optional Prop1, Optional Prop2)
	my ($FluidName,$InpCode,$Units,$Input1,$Input2) = @_;
	my (@xlkg,@xvkg,$qkg,$wliq,$wvap,$quality_value);
	our $QMASSdll;
	&CalcProp($FluidName, $InpCode, $Units, $Input1, $Input2);
	return &Trim_Then_Append($herr,"#") if ($ierr > 0);
	$quality_value = $q;
	$quality_value = &Trim_Then_Append("Not calculated","#") if ($q == 990);
	$quality_value = &Trim_Then_Append("Superheated vapor","#") if (($q == 998) || ($q > 1));
	$quality_value = &Trim_Then_Append("Supercritical state (T>Tc, p>pc)","#") if ($q == 999);
	$quality_value = &Trim_Then_Append("Subcooled liquid","#") if ($q < 0);
	$quality_value = &Trim_Then_Append("Subcooled liquid with p>pc","#") if ($q == -998);
	if((($q > 0) && ($q < 1)) && ($molmass == 2)){
		($q, $xliq[1], $xvap[1], $qkg, $xlkg[1], $xvkg[1], $wliq, $wvap, $ierr, $herr) = (pack('d',$q), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$qkg), pack('d',$xlkg[1]), pack('d',$xvkg[1]), pack('d',$wliq), pack('d',$wvap), pack('L!',$ierr), pack('A255',$herr));
		$QMASSdll->Call($q, $xliq[1], $xvap[1], $qkg, $xlkg[1], $xvkg[1], $wliq, $wvap, $ierr, $herr, pack('L!',255));
		($q, $xliq[1], $xvap[1], $qkg, $xlkg[1], $xvkg[1], $wliq, $wvap, $ierr, $herr) = (unpack('d',$q), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$qkg), unpack('d',$xlkg[1]), unpack('d',$xvkg[1]), unpack('d',$wliq), unpack('d',$wvap), unpack('L!',$ierr), unpack('A255',$herr));
		$quality_value = $qkg;
	}	
	return $quality_value;
};

$triggers{'temperature'} = $Func_Temperature;
$triggers{'pressure'} = $Func_Pressure;
$triggers{'density'} = $Func_Density;
$triggers{'cp'} = $Func_Cp;
$triggers{'enthalpy'} = $Func_Enthalpy;
$triggers{'entropy'} = $Func_Entropy;
$triggers{'volume'} = $Func_Volume;
$triggers{'viscosity'} = $Func_Viscosity;
$triggers{'quality'} = $Func_Quality;
@AVAIL_PROPERTIES = sort keys %triggers;

#TESTS#############################
Load_REFPROP(
	dll => 'C:\\Program Files\\RefpropMini\\REFPROP.DLL',
	fluids => 'C:\\Program Files\\RefpropMini\\fluids\\',
	mixtures => 'C:\\Program Files\\RefpropMini\\mixtures\\',
);
print "TEST1\n", "\nRefProp data files (DLL, fluids, mixtures) found in:\n", join("\n",@REF_DATA_LOCS), "\n\n";
print "TEST2\n", "\nRefProp Functions in DLL:\n", join("\n", @AVAIL_REF_FUNCS),"\n\n";
print "TEST3\n", "\nThis package can currently provide the following properties:\n", join("\n", @AVAIL_PROPERTIES),"\n\n";
print "TEST4\n", Get_Fluid_Property(property => 'density', fluid => "WATER",parameters => "TLIQ",units => "SI with C",input1 => 14.0), " kg/m^3\n\n";
print "TEST5\n", Get_Fluid_Property(property => 'viscosity', fluid => "WATER",parameters => "TLIQ",units => "SI with C",input1 => 14.0)*0.000001, " uPa-s\n\n";
print "TEST6\n", Get_Fluid_Property(property => 'cp', fluid => "WATER",parameters => "TLIQ",units => "SI with C",input1 => 14.0), " kJ/kg-K\n\n";
print "TEST7\n", Get_Fluid_Property(property => 'pressure', fluid => "R134A",parameters => "TVAP",units => "SI with C",input1 => 5.0)*1000, " MPa\n\n";
print "TEST8\n", Get_Fluid_Property(property => 'temperature', fluid => "R134A",parameters => "PVAP",units => "SI with C",input1 => (349.7 / 1000)), " deg C\n\n";
print "TEST9\n", Get_Fluid_Property(property => 'enthalpy', fluid => "R134A",parameters => "TP",units => "SI with C",input1 => 10.0,input2 => (349.7 / 1000)), " kJ/kg\n\n";
print "TEST10\n", Get_Fluid_Property(property => 'volume', fluid => "R134A",parameters => "TP",units => "SI with C",input1 => 10.0,input2 => (349.7 / 1000)), " m^3/kg\n\n";
print "TEST11\n", Get_Fluid_Property(property => 'entropy', fluid => "R134A",parameters => "TP",units => "SI with C",input1 => 10.0,input2 => (349.7 / 1000)), " kJ/kg-K\n\n";
print "TEST12\n", Get_Fluid_Property(property => 'quality', fluid => "R134A",parameters => "PH",units => "SI with C",input1 => (349.7 / 1000),input2 => 287.50), "\n\n";
print "TEST13\n", "R134A;0.5;R22;0.5 gives ", Get_Fluid_Property(property => 'cp', fluid => "R134A;0.5;R22;0.5 mass",parameters => "TVAP",units => "SI with C",input1 => -2.0), " kJ/kg-K\n\n";



#PUBLIC METHODS########################################################################################################################################

sub Load_REFPROP{
	my %data_locations = @_;
	my $CurrentShellDirectory = cwd();
	my ($ScriptName, $ScriptDirectory) = fileparse($0);
	our $RefPropDLL;         #Full path to .DLL
	our $FluidsDirectory;       #Full path to fluids directory
	our $MixturesDirectory;   #Full path to mixtures directory
	my $RefPropEnvironVarPath;
		
	$CurrentShellDirectory =~ s:/:\\:g;	
	$CurrentShellDirectory = $CurrentShellDirectory . '\\' unless (substr($CurrentShellDirectory,-1) eq '\\');
	$ScriptDirectory = $ScriptDirectory . '\\' unless (substr($ScriptDirectory,-1) eq '\\');
	
	#Check that REFPROP.DLL can be located	
	if(exists($data_locations{dll})){
		$RefPropDLL = $data_locations{dll};
		$RefPropDLL =~ s:/:\\:g;
		$RefPropDLL = substr($RefPropDLL,1) if ((($RefPropDLL =~ m/\\/) == 1) && (substr($RefPropDLL,0,1) eq "\\")); 		
	}else{
		$RefPropDLL = "REFPROP.DLL";
	}
	unless($RefPropDLL =~ m/\\/){
		if(-e ($CurrentShellDirectory . $RefPropDLL)){
			$RefPropDLL = $CurrentShellDirectory . $RefPropDLL;
		}elsif(-e ($ScriptDirectory . $RefPropDLL)){
			$RefPropDLL = $ScriptDirectory . $RefPropDLL;
		}elsif(exists($ENV{"RPPrefix"})){
			$RefPropEnvironVarPath = &Trim_Then_Append($ENV{"RPPrefix"});
			$RefPropEnvironVarPath = substr($RefPropEnvironVarPath,0,-1) if (substr($RefPropEnvironVarPath,-1) eq ";");
			$RefPropEnvironVarPath = $RefPropEnvironVarPath . '\\' unless (substr($RefPropEnvironVarPath,-1) eq '\\');
			$RefPropDLL = $RefPropEnvironVarPath . $RefPropDLL if (-e ($RefPropEnvironVarPath . $RefPropDLL));
		}elsif(-e ("C:\\Program Files\\Refprop\\" . $RefPropDLL)){
			$RefPropDLL = "C:\\Program Files\\Refprop\\" . $RefPropDLL;
		}else{die "$RefPropDLL not found in current shell directory, executable directory, RPPrefix directory or within C:\\Program Files\\Refprop\\, ensure that the file exists!\n";}
	}else{
		if(not -e $RefPropDLL){die '"' . $RefPropDLL . '"' . " not found, ensure that the file exists.\n";}
	}
	my ($DLLFilename,$DLLDirectory) = fileparse($RefPropDLL);
	$DLLDirectory = $DLLDirectory . '\\' unless (substr($DLLDirectory,-1) eq '\\');
	if(exists($data_locations{fluids})){
		$FluidsDirectory = $data_locations{fluids};
		$FluidsDirectory =~ s:/:\\:g;
		$FluidsDirectory = $FluidsDirectory . '\\' unless (substr($FluidsDirectory,-1) eq '\\');
		if(not -e $FluidsDirectory){die '"' . $FluidsDirectory . '"' . " not found, ensure that the directory exists.\n";}	
	}else{
		$FluidsDirectory = "fluids\\";
		if(-e ($CurrentShellDirectory . $FluidsDirectory)){
			$FluidsDirectory = $CurrentShellDirectory . $FluidsDirectory;
		}elsif(-e ($ScriptDirectory . $FluidsDirectory)){
			$FluidsDirectory = $ScriptDirectory . $FluidsDirectory;
		}elsif(exists($ENV{"RPPrefix"})){
			$RefPropEnvironVarPath = &Trim_Then_Append($ENV{"RPPrefix"});
			$RefPropEnvironVarPath = substr($RefPropEnvironVarPath,0,-1) if (substr($RefPropEnvironVarPath,-1) eq ";");
			$RefPropEnvironVarPath = $RefPropEnvironVarPath . '\\' unless (substr($RefPropEnvironVarPath,-1) eq '\\');
			$FluidsDirectory = $RefPropEnvironVarPath . $FluidsDirectory if (-e ($RefPropEnvironVarPath . $FluidsDirectory));
		}elsif(-e ("C:\\Program Files\\Refprop\\" . $FluidsDirectory)){
			$FluidsDirectory = "C:\\Program Files\\Refprop\\" . $FluidsDirectory;
		}elsif(-e ($DLLDirectory . $FluidsDirectory)){
			$FluidsDirectory = $DLLDirectory . $FluidsDirectory;
		}else{die "$FluidsDirectory not found in current shell directory, executable directory, RPPrefix directory, C:\\Program Files\\Refprop\\ or within the same directory as the Refprop library ($DLLDirectory), ensure that the directory exists!\n";}
	}
	if(exists($data_locations{mixtures})){
		$MixturesDirectory = $data_locations{mixtures};
		$MixturesDirectory =~ s:/:\\:g;
		$MixturesDirectory = $MixturesDirectory . '\\' unless (substr($MixturesDirectory,-1) eq '\\');
		if(not -e $MixturesDirectory){die '"' . $MixturesDirectory . '"' . " not found, ensure that the directory exists.\n";}	
	}else{
		$MixturesDirectory = "mixtures\\";
		if(-e ($CurrentShellDirectory . $MixturesDirectory)){
			$MixturesDirectory = $CurrentShellDirectory . $MixturesDirectory;
		}elsif(-e ($ScriptDirectory . $MixturesDirectory)){
			$MixturesDirectory = $ScriptDirectory . $MixturesDirectory;
		}elsif(exists($ENV{"RPPrefix"})){
			$RefPropEnvironVarPath = &Trim_Then_Append($ENV{"RPPrefix"});
			$RefPropEnvironVarPath = substr($RefPropEnvironVarPath,0,-1) if (substr($RefPropEnvironVarPath,-1) eq ";");
			$RefPropEnvironVarPath = $RefPropEnvironVarPath . '\\' unless (substr($RefPropEnvironVarPath,-1) eq '\\');
			$MixturesDirectory = $RefPropEnvironVarPath . $MixturesDirectory if (-e ($RefPropEnvironVarPath . $MixturesDirectory));
		}elsif(-e ("C:\\Program Files\\Refprop\\" . $MixturesDirectory)){
			$MixturesDirectory = "C:\\Program Files\\Refprop\\" . $MixturesDirectory;
		}elsif(-e ($DLLDirectory . $MixturesDirectory)){
			$MixturesDirectory = $DLLDirectory . $MixturesDirectory;
		}else{die "$MixturesDirectory not found in current shell directory, executable directory, RPPrefix directory, C:\\Program Files\\Refprop\\ or within the same directory as the Refprop library ($DLLDirectory), ensure that the directory exists!\n";}
	}
	our @REF_DATA_LOCS = ();
	@REF_DATA_LOCS = ($RefPropDLL, $FluidsDirectory, $MixturesDirectory);
	

	#REFPROP9 DLL FUNCTIONS - Referenced from VBA Code version 9.036 found at http://www.boulder.nist.gov/div838/theory/refprop/REFPROP.XLS (last updated Jan. 16, 2012)

	#'ABFL1dll'[Double, Double, Double, Long, String, Double, Double, Double, Double, Double, Long, String, Long, Long]
	#'ABFL2dll'[Double, Double, Double, Long, Long, String, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long, Long]
	#'ACTVYdll'[Double, Double, Double, Double, Double, Long, String, Long]
	#'AGdll'[Double, Double, Double, Double, Double]
	#'CCRITdll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'CHEMPOTdll'[Double, Double, Double, Double, Long, String, Long]
	#'CRITPdll'[Double, Double, Double, Double, Long, String, Long]
	#'CSATKdll'[Long, Double, Long, Double, Double, Double, Long, String, Long]
	#'CSTARdll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'CV2PKdll'[Long, Double, Double, Double, Double, Long, String, Long]
	#'CVCPdll'[Double, Double, Double, Double, Double]
	#'DBDTdll'[Double, Double, Double]
	#'DBFL2dll'[Double, Double, Double, Long, String, Double, Double, Double, Double, Double, Double, Double, Long, String, Long, Long]
	#'DDDPdll'[Double, Double, Double, Double]
	#'DDDTdll'[Double, Double, Double, Double]
	#'DEFLSHdll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'DHD1dll'[Double, Double, Double, Double, Double, Double, Double, Double, Double]
	#'DHFLSHdll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'DIELECdll'[Double, Double, Double, Double]
	#'DPDD2dll'[Double, Double, Double, Double]
	#'DPDDdll'[Double, Double, Double, Double]
	#'DPDTdll'[Double, Double, Double, Double]
	#'DPTSATKdll'[Long, Double, Long, Double, Double, Double, Double, Long, String, Long]
	#'DSFLSHdll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'ENTHALdll'[Double, Double, Double, Double]
	#'ENTROdll'[Double, Double, Double, Double]
	#'ESFLSHdll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'ETAK0dll'[Long, Double, Double, Long, String, Long]
	#'ETAK1dll'[Long, Double, Double, Long, String, Long]
	#'ETAKBdll'[Long, Double, Double, Double, Long, String, Long]
	#'ETAKRdll'[Long, Double, Double, Double, Long, String, Long]
	#'FGCTYdll'[Double, Double, Double, Double]
	#'FPVdll'[Double, Double, Double, Double, Double]
	#'FUGCOFdll'[Double, Double, Double, Double, Long, String, Long]
	#'GERG04dll'[Long, Long, Long, String, Long]
	#'GETFIJdll'[String, Double, String, String, Long, Long, Long]
	#'GETKTVdll'[Long, Long, String, Double, String, String, String, String, Long, Long, Long, Long, Long]
	#'HEATdll'[Double, Double, Double, Double, Double, Long, String, Long]
	#'HSFLSHdll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'INFOdll'[Long, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double]
	#'LIMITKdll'[String, Long, Double, Double, Double, Double, Double, Double, Double, Long, String, Long, Long]
	#'LIMITXdll'[String, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long, Long]
	#'MAXPdll'[Double, Double, Double, Double, Long, String, Long]
	#'MAXTdll'[Double, Double, Double, Double, Long, String, Long]
	#'MELTPdll'[Double, Double, Double, Long, String, Long]
	#'MELTTdll'[Double, Double, Double, Long, String, Long]
	#'MLTH2Odll'[Double, Double, Double]
	#'OMEGAdll'[Long, Double, Double, Double]
	#'PDFLSHdll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'PEFLSHdll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'PHFLSHdll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'PQFLSHdll'[Double, Double, Double, Long, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'PREOSdll'[Long]
	#'PRESSdll'[Double, Double, Double, Double]
	#'PSFLSHdll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'PUREFLDdll'[Long]
	#'QMASSdll'[Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'QMOLEdll'[Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'SATDATAdll'[Double]
	#'SATDdll'[Double, Double, Long, Long, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'SATEdll'[Double, Double, Long, Long, Long, Double, Double, Double, Long, Double, Double, Double, Long, String, Long]
	#'SATHdll'[Double, Double, Long, Long, Long, Double, Double, Double, Long, Double, Double, Double, Long, String, Long]
	#'SATPdll'[Double, Double, Long, Double, Double, Double, Double, Double, Long, String, Long]
	#'SATSdll'[Double, Double, Long, Long, Long, Double, Double, Double, Long, Double, Double, Double, Long, Double, Double, Double, Long, String, Long]
	#'SATTdll'[Double, Double, Long, Double, Double, Double, Double, Double, Long, String, Long]
	#'SATTPdll'[Double, Double, Double, Long, Long, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'SETAGAdll'[Long, String, Long]
	#'SETKTVdll'[Long, Long, String, Double, String, Long, String, Long, Long, Long]
	#'SETMIXdll'[String, String, String, Long, String, Double, Long, String, Long, Long, Long, Long, Long]
	#'SETMODdll'[Long, String, String, String, Long, String, Long, Long, Long, Long]
	#'SETREFdll'[String, Long, Double, Double, Double, Double, Double, Long, String, Long, Long]
	#'SETUPdll'[Long, String, String, String, Long, String, Long, Long, Long, Long]
	##'SPECGRdll'[Double, Double, Double, Double]
	#'SUBLPdll'[Double, Double, Double, Long, String, Long]
	#'SUBLTdll'[Double, Double, Double, Long, String, Long]
	#'SURFTdll'[Double, Double, Double, Double, Long, String, Long]
	#'SURTENdll'[Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'TCXK0dll'[Long, Double, Double, Long, String, Long]
	#'TCXKBdll'[Long, Double, Double, Double, Long, String, Long]
	#'TCXKCdll'[Long, Double, Double, Double, Long, String, Long]
	#'TDFLSHdll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'TEFLSHdll'[Double, Double, Double, Long, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'THERM0dll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double]
	#'THERM2dll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double]
	#'THERM3dll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double]
	#'THERMdll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double]
	#'THFLSHdll'[Double, Double, Double, Long, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'TPFLSHdll'[Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'TPRHOdll'[Double, Double, Double, Long, Long, Double, Long, String, Long]
	#'TQFLSHdll'[Double, Double, Double, Long, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'TRNPRPdll'[Double, Double, Double, Double, Double, Long, String, Long]
	#'TSFLSHdll'[Double, Double, Double, Long, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long]
	#'UNSETAGAdll'[]
	#'VIRBdll'[Double, Double, Double]
	#'VIRCdll'[Double, Double, Double]
	#'WMOLdll'[Double, Double]
	#'XMASSdll'[Double, Double, Double]
	#'XMOLEdll'[Double, Double, Double]

	#Beginning loading functions from REFPROP for use: NOTE - All data sent to the DLL set up as Pointer ('P') so that they can be changed in memory. These functions
	#														do not return any values, so return set to empty/void ('V').

	our ($ABFL1dll,$ABFL2dll,$ACTVYdll,$AGdll,$CCRITdll,$CHEMPOTdll,$CRITPdll,$CSATKdll,$CSTARdll,$CV2PKdll,$CVCPdll);
	our ($DBDTdll,$DBFL2dll,$DDDPdll,$DDDTdll,$DEFLSHdll,$DHD1dll,$DHFLSHdll,$DIELECdll,$DPDD2dll,$DPDDdll,$DPDTdll);
	our ($DPTSATKdll,$DSFLSHdll,$ENTHALdll,$ENTROdll,$ESFLSHdll,$ETAK0dll,$ETAK1dll,$ETAKBdll,$ETAKRdll,$FGCTYdll,$FPVdll);
	our ($FUGCOFdll,$GERG04dll,$GETFIJdll,$GETKTVdll,$HEATdll,$HSFLSHdll,$INFOdll,$LIMITKdll,$LIMITXdll,$MAXPdll,$MAXTdll);	
	our ($MELTPdll,$MELTTdll,$MLTH2Odll,$OMEGAdll,$PDFLSHdll,$PEFLSHdll,$PHFLSHdll,$PQFLSHdll,$PREOSdll,$PRESSdll,$PSFLSHdll);
	our ($PUREFLDdll,$QMASSdll,$QMOLEdll,$SATDATAdll,$SATDdll,$SATEdll,$SATHdll,$SATPdll,$SATSdll,$SATTdll,$SATTPdll,$SETAGAdll);
	our ($SETKTVdll,$SETMIXdll,$SETMODdll,$SETREFdll,$SETUPdll,$SPECGRdll,$SUBLPdll,$SUBLTdll,$SURFTdll,$SURTENdll,$TCXK0dll);
	our ($TCXKBdll,$TCXKCdll,$TDFLSHdll,$TEFLSHdll,$THERM0dll,$THERM2dll,$THERM3dll,$THERMdll,$THFLSHdll,$TPFLSHdll,$TPRHOdll);
	our ($TQFLSHdll,$TRNPRPdll,$TSFLSHdll,$UNSETAGAdll,$VIRBdll,$VIRCdll,$WMOLdll,$XMASSdll,$XMOLEdll);
	our @AVAIL_REF_FUNCS = ();
	
	$ABFL1dll = Win32::API->new(
		$RefPropDLL,'ABFL1dll','PPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Long, String, Double, Double, Double, Double, Double, Long, String, Long, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'ABFL1dll' if $ABFL1dll;

	$ABFL2dll = Win32::API->new(
		$RefPropDLL,'ABFL2dll','PPPPPPPPPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Long, Long, String, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'ABFL2dll' if $ABFL2dll;

	$ACTVYdll = Win32::API->new(
		$RefPropDLL,'ACTVYdll','PPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'ACTVYdll' if $ACTVYdll;

	$AGdll = Win32::API->new(
		$RefPropDLL,'AGdll','PPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'AGdll' if $AGdll;

	$CCRITdll = Win32::API->new(
		$RefPropDLL,'CCRITdll','PPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'CCRITdll' if $CCRITdll;

	$CHEMPOTdll = Win32::API->new(
		$RefPropDLL,'CHEMPOTdll','PPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'CHEMPOTdll' if $CHEMPOTdll;

	$CRITPdll = Win32::API->new(
		$RefPropDLL,'CRITPdll','PPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'CRITPdll' if $CRITPdll;

	$CSATKdll = Win32::API->new(
		$RefPropDLL,'CSATKdll','PPPPPPPPP','V',
	);   #Takes arguments of (Long, Double, Long, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'CSATKdll' if $CSATKdll;

	$CSTARdll = Win32::API->new(
		$RefPropDLL,'CSTARdll','PPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'CSTARdll' if $CSTARdll;

	$CV2PKdll = Win32::API->new(
		$RefPropDLL,'CV2PKdll','PPPPPPPP','V',
	);   #Takes arguments of (Long, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'CV2PKdll' if $CV2PKdll;

	$CVCPdll = Win32::API->new(
		$RefPropDLL,'CVCPdll','PPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'CVCPdll' if $CVCPdll;

	$DBDTdll = Win32::API->new(
		$RefPropDLL,'DBDTdll','PPP','V',
	);   #Takes arguments of (Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'DBDTdll' if $DBDTdll;

	$DBFL2dll = Win32::API->new(
		$RefPropDLL,'DBFL2dll','PPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Long, String, Double, Double, Double, Double, Double, Double, Double, Long, String, Long, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'DBFL2dll' if $DBFL2dll;

	$DDDPdll = Win32::API->new(
		$RefPropDLL,'DDDPdll','PPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'DDDPdll' if $DDDPdll;

	$DDDTdll = Win32::API->new(
		$RefPropDLL,'DDDTdll','PPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'DDDTdll' if $DDDTdll;

	$DEFLSHdll = Win32::API->new(
		$RefPropDLL,'DEFLSHdll','PPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'DEFLSHdll' if $DEFLSHdll;

	$DHD1dll = Win32::API->new(
		$RefPropDLL,'DHD1dll','PPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'DHD1dll' if $DHD1dll;

	$DHFLSHdll = Win32::API->new(
		$RefPropDLL,'DHFLSHdll','PPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'DHFLSHdll' if $DHFLSHdll;

	$DIELECdll = Win32::API->new(
		$RefPropDLL,'DIELECdll','PPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'DIELECdll' if $DIELECdll;

	$DPDD2dll = Win32::API->new(
		$RefPropDLL,'DPDD2dll','PPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'DPDD2dll' if $DPDD2dll;

	$DPDDdll = Win32::API->new(
		$RefPropDLL,'DPDDdll','PPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'DPDDdll' if $DPDDdll;

	$DPDTdll = Win32::API->new(
		$RefPropDLL,'DPDTdll','PPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'DPDTdll' if $DPDTdll;

	$DPTSATKdll = Win32::API->new(
		$RefPropDLL,'DPTSATKdll','PPPPPPPPPP','V',
	);   #Takes arguments of (Long, Double, Long, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'DPTSATKdll' if $DPTSATKdll;

	$DSFLSHdll = Win32::API->new(
		$RefPropDLL,'DSFLSHdll','PPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'DSFLSHdll' if $DSFLSHdll;

	$ENTHALdll = Win32::API->new(
		$RefPropDLL,'ENTHALdll','PPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'ENTHALdll' if $ENTHALdll;

	$ENTROdll = Win32::API->new(
		$RefPropDLL,'ENTROdll','PPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'ENTROdll' if $ENTROdll;

	$ESFLSHdll = Win32::API->new(
		$RefPropDLL,'ESFLSHdll','PPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'ESFLSHdll' if $ESFLSHdll;

	$ETAK0dll = Win32::API->new(
		$RefPropDLL,'ETAK0dll','PPPPPP','V',
	);   #Takes arguments of (Long, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'ETAK0dll' if $ETAK0dll;

	$ETAK1dll = Win32::API->new(
		$RefPropDLL,'ETAK1dll','PPPPPP','V',
	);   #Takes arguments of (Long, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'ETAK1dll' if $ETAK1dll;

	$ETAKBdll = Win32::API->new(
		$RefPropDLL,'ETAKBdll','PPPPPPP','V',
	);   #Takes arguments of (Long, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'ETAKBdll' if $ETAKBdll;

	$ETAKRdll = Win32::API->new(
		$RefPropDLL,'ETAKRdll','PPPPPPP','V',
	);   #Takes arguments of (Long, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'ETAKRdll' if $ETAKRdll;

	$FGCTYdll = Win32::API->new(
		$RefPropDLL,'FGCTYdll','PPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'FGCTYdll' if $FGCTYdll;

	$FPVdll = Win32::API->new(
		$RefPropDLL,'FPVdll','PPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'FPVdll' if $FPVdll;

	$FUGCOFdll = Win32::API->new(
		$RefPropDLL,'FUGCOFdll','PPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'FUGCOFdll' if $FUGCOFdll;

	$GERG04dll = Win32::API->new(
		$RefPropDLL,'GERG04dll','PPPPP','V',
	);   #Takes arguments of (Long, Long, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'GERG04dll' if $GERG04dll;

	$GETFIJdll = Win32::API->new(
		$RefPropDLL,'GETFIJdll','PPPPPPP','V',
	);   #Takes arguments of (String, Double, String, String, Long, Long, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'GETFIJdll' if $GETFIJdll;

	$GETKTVdll = Win32::API->new(
		$RefPropDLL,'GETKTVdll','PPPPPPPPPPPPP','V',
	);   #Takes arguments of (Long, Long, String, Double, String, String, String, String, Long, Long, Long, Long, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'GETKTVdll' if $GETKTVdll;

	$HEATdll = Win32::API->new(
		$RefPropDLL,'HEATdll','PPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'HEATdll' if $HEATdll;

	$HSFLSHdll = Win32::API->new(
		$RefPropDLL,'HSFLSHdll','PPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'HSFLSHdll' if $HSFLSHdll;

	$INFOdll = Win32::API->new(
		$RefPropDLL,'INFOdll','PPPPPPPPPPP','V',
	);   #Takes arguments of (Long, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'INFOdll' if $INFOdll;

	$LIMITKdll = Win32::API->new(
		$RefPropDLL,'LIMITKdll','PPPPPPPPPPPPP','V',
	);   #Takes arguments of (String, Long, Double, Double, Double, Double, Double, Double, Double, Long, String, Long, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'LIMITKdll' if $LIMITKdll;

	$LIMITXdll = Win32::API->new(
		$RefPropDLL,'LIMITXdll','PPPPPPPPPPPPP','V',
	);   #Takes arguments of (String, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'LIMITXdll' if $LIMITXdll;

	$MAXPdll = Win32::API->new(
		$RefPropDLL,'MAXPdll','PPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'MAXPdll' if $MAXPdll;

	$MAXTdll = Win32::API->new(
		$RefPropDLL,'MAXTdll','PPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'MAXTdll' if $MAXTdll;

	$MELTPdll = Win32::API->new(
		$RefPropDLL,'MELTPdll','PPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'MELTPdll' if $MELTPdll;

	$MELTTdll = Win32::API->new(
		$RefPropDLL,'MELTTdll','PPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'MELTTdll' if $MELTTdll;

	$MLTH2Odll = Win32::API->new(
		$RefPropDLL,'MLTH2Odll','PPP','V',
	);   #Takes arguments of (Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'MLTH2Odll' if $MLTH2Odll;

	$OMEGAdll = Win32::API->new(
		$RefPropDLL,'OMEGAdll','PPPP','V',
	);   #Takes arguments of (Long, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'OMEGAdll' if $OMEGAdll;

	$PDFLSHdll = Win32::API->new(
		$RefPropDLL,'PDFLSHdll','PPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'PDFLSHdll' if $PDFLSHdll;

	$PEFLSHdll = Win32::API->new(
		$RefPropDLL,'PEFLSHdll','PPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'PEFLSHdll' if $PEFLSHdll;

	$PHFLSHdll = Win32::API->new(
		$RefPropDLL,'PHFLSHdll','PPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'PHFLSHdll' if $PHFLSHdll;

	$PQFLSHdll = Win32::API->new(
		$RefPropDLL,'PQFLSHdll','PPPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Long, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'PQFLSHdll' if $PQFLSHdll;

	$PREOSdll = Win32::API->new(
		$RefPropDLL,'PREOSdll','P','V',
	);   #Takes arguments of (Long) and returns nothing
	push @AVAIL_REF_FUNCS,'PREOSdll' if $PREOSdll;

	$PRESSdll = Win32::API->new(
		$RefPropDLL,'PRESSdll','PPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'PRESSdll' if $PRESSdll;

	$PSFLSHdll = Win32::API->new(
		$RefPropDLL,'PSFLSHdll','PPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'PSFLSHdll' if $PSFLSHdll;

	$PUREFLDdll = Win32::API->new(
		$RefPropDLL,'PUREFLDdll','P','V',
	);   #Takes arguments of (Long) and returns nothing
	push @AVAIL_REF_FUNCS,'PUREFLDdll' if $PUREFLDdll;

	$QMASSdll = Win32::API->new(
		$RefPropDLL,'QMASSdll','PPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'QMASSdll' if $QMASSdll;

	$QMOLEdll = Win32::API->new(
		$RefPropDLL,'QMOLEdll','PPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'QMOLEdll' if $QMOLEdll;

	$SATDATAdll = Win32::API->new(
		$RefPropDLL,'SATDATAdll','P','V',
	);   #Takes arguments of (Double) and returns nothing
	push @AVAIL_REF_FUNCS,'SATDATAdll' if $SATDATAdll;

	$SATDdll = Win32::API->new(
		$RefPropDLL,'SATDdll','PPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Long, Long, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SATDdll' if $SATDdll;

	$SATEdll = Win32::API->new(
		$RefPropDLL,'SATEdll','PPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Long, Long, Long, Double, Double, Double, Long, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SATEdll' if $SATEdll;

	$SATHdll = Win32::API->new(
		$RefPropDLL,'SATHdll','PPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Long, Long, Long, Double, Double, Double, Long, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SATHdll' if $SATHdll;

	$SATPdll = Win32::API->new(
		$RefPropDLL,'SATPdll','PPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Long, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SATPdll' if $SATPdll;

	$SATSdll = Win32::API->new(
		$RefPropDLL,'SATSdll','PPPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Long, Long, Long, Double, Double, Double, Long, Double, Double, Double, Long, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SATSdll' if $SATSdll;

	$SATTdll = Win32::API->new(
		$RefPropDLL,'SATTdll','PPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Long, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SATTdll' if $SATTdll;

	$SATTPdll = Win32::API->new(
		$RefPropDLL,'SATTPdll','PPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Long, Long, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SATTPdll' if $SATTPdll;

	$SETAGAdll = Win32::API->new(
		$RefPropDLL,'SETAGAdll','PPP','V',
	);   #Takes arguments of (Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SETAGAdll' if $SETAGAdll;

	$SETKTVdll = Win32::API->new(
		$RefPropDLL,'SETKTVdll','PPPPPPPPPP','V',
	);   #Takes arguments of (Long, Long, String, Double, String, Long, String, Long, Long, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SETKTVdll' if $SETKTVdll;

	$SETMIXdll = Win32::API->new(
		$RefPropDLL,'SETMIXdll','PPPPPPPPPPPPP','V',
	);   #Takes arguments of (String, String, String, Long, String, Double, Long, String, Long, Long, Long, Long, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SETMIXdll' if $SETMIXdll;

	$SETMODdll = Win32::API->new(
		$RefPropDLL,'SETMODdll','PPPPPPPPPP','V',
	);   #Takes arguments of (Long, String, String, String, Long, String, Long, Long, Long, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SETMODdll' if $SETMODdll;

	$SETREFdll = Win32::API->new(
		$RefPropDLL,'SETREFdll','PPPPPPPPPPP','V',
	);   #Takes arguments of (String, Long, Double, Double, Double, Double, Double, Long, String, Long, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SETREFdll' if $SETREFdll;

	$SETUPdll = Win32::API->new(
		$RefPropDLL,'SETUPdll','PPPPPPPPPP','V',
	);   #Takes arguments of (Long, String, String, String, Long, String, Long, Long, Long, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SETUPdll' if $SETUPdll;

	$SPECGRdll = Win32::API->new(
		$RefPropDLL,'SPECGRdll','PPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'SPECGRdll' if $SPECGRdll;

	$SUBLPdll = Win32::API->new(
		$RefPropDLL,'SUBLPdll','PPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SUBLPdll' if $SUBLPdll;

	$SUBLTdll = Win32::API->new(
		$RefPropDLL,'SUBLTdll','PPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SUBLTdll' if $SUBLTdll;

	$SURFTdll = Win32::API->new(
		$RefPropDLL,'SURFTdll','PPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SURFTdll' if $SURFTdll;

	$SURTENdll = Win32::API->new(
		$RefPropDLL,'SURTENdll','PPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'SURTENdll' if $SURTENdll;

	$TCXK0dll = Win32::API->new(
		$RefPropDLL,'TCXK0dll','PPPPPP','V',
	);   #Takes arguments of (Long, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'TCXK0dll' if $TCXK0dll;

	$TCXKBdll = Win32::API->new(
		$RefPropDLL,'TCXKBdll','PPPPPPP','V',
	);   #Takes arguments of (Long, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'TCXKBdll' if $TCXKBdll;

	$TCXKCdll = Win32::API->new(
		$RefPropDLL,'TCXKCdll','PPPPPPP','V',
	);   #Takes arguments of (Long, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'TCXKCdll' if $TCXKCdll;

	$TDFLSHdll = Win32::API->new(
		$RefPropDLL,'TDFLSHdll','PPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'TDFLSHdll' if $TDFLSHdll;

	$TEFLSHdll = Win32::API->new(
		$RefPropDLL,'TEFLSHdll','PPPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Long, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'TEFLSHdll' if $TEFLSHdll;

	$THERM0dll = Win32::API->new(
		$RefPropDLL,'THERM0dll','PPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'THERM0dll' if $THERM0dll;

	$THERM2dll = Win32::API->new(
		$RefPropDLL,'THERM2dll','PPPPPPPPPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'THERM2dll' if $THERM2dll;

	$THERM3dll = Win32::API->new(
		$RefPropDLL,'THERM3dll','PPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'THERM3dll' if $THERM3dll;

	$THERMdll = Win32::API->new(
		$RefPropDLL,'THERMdll','PPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'THERMdll' if $THERMdll;

	$THFLSHdll = Win32::API->new(
		$RefPropDLL,'THFLSHdll','PPPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Long, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'THFLSHdll' if $THFLSHdll;

	$TPFLSHdll = Win32::API->new(
		$RefPropDLL,'TPFLSHdll','PPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'TPFLSHdll' if $TPFLSHdll;

	$TPRHOdll = Win32::API->new(
		$RefPropDLL,'TPRHOdll','PPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Long, Long, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'TPRHOdll' if $TPRHOdll;

	$TQFLSHdll = Win32::API->new(
		$RefPropDLL,'TQFLSHdll','PPPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Long, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'TQFLSHdll' if $TQFLSHdll;

	$TRNPRPdll = Win32::API->new(
		$RefPropDLL,'TRNPRPdll','PPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'TRNPRPdll' if $TRNPRPdll;

	$TSFLSHdll = Win32::API->new(
		$RefPropDLL,'TSFLSHdll','PPPPPPPPPPPPPPPPPPP','V',
	);   #Takes arguments of (Double, Double, Double, Long, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Double, Long, String, Long) and returns nothing
	push @AVAIL_REF_FUNCS,'TSFLSHdll' if $TSFLSHdll;

	$UNSETAGAdll = Win32::API->new(
		$RefPropDLL,'UNSETAGAdll',[],'V',
	);   #Takes arguments of () and returns nothing
	push @AVAIL_REF_FUNCS,'UNSETAGAdll' if $UNSETAGAdll;

	$VIRBdll = Win32::API->new(
		$RefPropDLL,'VIRBdll','PPP','V',
	);   #Takes arguments of (Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'VIRBdll' if $VIRBdll;

	$VIRCdll = Win32::API->new(
		$RefPropDLL,'VIRCdll','PPP','V',
	);   #Takes arguments of (Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'VIRCdll' if $VIRCdll;

	$WMOLdll = Win32::API->new(
		$RefPropDLL,'WMOLdll','PP','V',
	);   #Takes arguments of (Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'WMOLdll' if $WMOLdll;

	$XMASSdll = Win32::API->new(
		$RefPropDLL,'XMASSdll','PPP','V',
	);   #Takes arguments of (Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'XMASSdll' if $XMASSdll;

	$XMOLEdll = Win32::API->new(
		$RefPropDLL,'XMOLEdll','PPP','V',
	);   #Takes arguments of (Double, Double, Double) and returns nothing
	push @AVAIL_REF_FUNCS,'XMOLEdll' if $XMOLEdll;

}
sub Get_Fluid_Property{
	my %argument_list = @_;
	my $Property;
	my $FluidName;
	my $InpCode;
	my $Units;
	my $Input1;
	my $Input2;
	
	return $Loading_Warning if ($REF_DATA_LOCS[0] eq $Loading_Warning);
	
	if(exists($argument_list{property})){
		$Property = $argument_list{property};
	}else{return "error: property must be entered!";}
	if(exists($argument_list{fluid})){
		$FluidName = $argument_list{fluid};
	}else{return "error: fluid must be entered!";}
	if(exists($argument_list{parameters})){
		$InpCode = $argument_list{parameters};
	}else{return "error: parameters must be entered!";}
	if(exists($argument_list{units})){
		$Units = $argument_list{units};
	}
	if(exists($argument_list{input1})){
		$Input1 = $argument_list{input1};
	}
	if(exists($argument_list{input2})){
		$Input2 = $argument_list{input2};
	}
	
	return &{$triggers{lc($argument_list{property})}}($FluidName, $InpCode, $Units, $Input1, $Input2);
}


#PRIVATE SUBS/FUNCTIONS/METHODS############################################################################################

sub Calc{	#(InputCode, Prop1, Prop2, iflag1, iflag2)
	my ($a,$Input1,$Input2); 	#VBA As String
	my (@xlkg,@xvkg,@xlj,@xvj,$qmol,$wliq,$wvap,$pp);	#VBA As Double
	my $i;	#VBA As Integer
	my ($InpCode);
	my ($InputCode, $Prop1, $Prop2, $iflag1, $iflag2) = @_;
	our ($SATTdll,$SATPdll,$SATDdll,$SATHdll,$SATSdll,$THERMdll,$TPRHOdll,$TPFLSHdll);
	our ($TDFLSHdll,$THFLSHdll,$TSFLSHdll,$TEFLSHdll,$TQFLSHdll,$XMASSdll, $QMOLEdll);
	our ($PDFLSHdll,$PHFLSHdll,$PSFLSHdll,$PEFLSHdll,$PQFLSHdll,$DHFLSHdll,$DSFLSHdll);
	our ($DEFLSHdll,$HSFLSHdll,$MELTTdll,$MELTPdll,$SUBLTdll,$SUBLPdll);
	$ierr = 0;
	$herr = "";
	$InpCode = &Trim_Then_Append(uc($InputCode));
	$Input2 = "";
	$Input1 = substr($InpCode,0,1);
	$Input2 = substr($InpCode,1,1) if (length($InpCode) == 2);
	if(length($InpCode) == 3){
		$a = substr($InpCode,-1);
		$Input2 = substr($InpCode,1,1) if (($a eq "&") || ($a eq "<") || ($a eq ">"));  	
	}
	$Input2 = substr($InpCode,1,1) if ((substr($InpCode,0,2) eq "TP") || (substr($InpCode,0,2) eq "PT"));
	if($Input1 eq "T"){
		$t = $Prop1;
		if($iflag1 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("First property missing","#");
			return;
		}
	}
	if($Input1 eq "P"){
		$p = $Prop1;
		if($iflag1 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("First property missing","#");
			return;
		}
	}
	if($Input1 eq "D"){
		$d = $Prop1;
		if($iflag1 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("First property missing","#");
			return;
		}
	}
	if((($Input1 eq "V") && ($Prop1 != 0)) && (length($InpCode) == 2)){
		$d = 1 / $Prop1;
		substr($InpCode,0,1) = "D";
		if($iflag1 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("First property missing","#");
			return;
		}
	}
	if($Input1 eq "E"){
		$e = $Prop1;
		if($iflag1 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("First property missing","#");
			return;
		}
	}
	if($Input1 eq "H"){
		$h = $Prop1;
		if($iflag1 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("First property missing","#");
			return;
		}
	}
	if($Input1 eq "S"){
		$s = $Prop1;
		if($iflag1 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("First property missing","#");
			return;
		}
	}
	if($Input1 eq "Q"){
		$q = $Prop1;
		if($iflag1 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("First property missing","#");
			return;
		}
	}
	if($Input2 eq "T"){
		$t = $Prop2;
		if($iflag2 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("Second property missing","#");
			return;
		}
	}
	if($Input2 eq "P"){
		$p = $Prop2;
		if($iflag2 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("Second property missing","#");
			return;
		}
	}
	if($Input2 eq "D"){
		$d = $Prop2;
		if($iflag2 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("Second property missing","#");
			return;
		}
	}
	if((($Input2 eq "V") && ($Prop2 != 0)) && (length($InpCode) == 2)){
		$d = 1 / $Prop2;
		substr($InpCode,1,1) = "D";
		if($iflag2 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("Second property missing","#");
			return;
		}
	}
	if($Input2 eq "E"){
		$e = $Prop2;
		if($iflag2 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("Second property missing","#");
			return;
		}
	}
	if($Input2 eq "H"){
		$h = $Prop2;
		if($iflag2 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("Second property missing","#");
			return;
		}
	}
	if($Input2 eq "S"){
		$s = $Prop2;
		if($iflag2 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("Second property missing","#");
			return;
		}
	}
	if($Input2 eq "Q"){
		$q = $Prop2;
		if($iflag2 >= 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("Second property missing","#");
			return;
		}
	}
	$phase = 2;
	$phase = 1 if((length($InpCode) > 1) && (uc(substr($InpCode,1,1)) eq "L"));
	foreach $i (1..$nc){
		($xliq[$i],$xvap[$i]) = (0,0);
	}
	if((substr($InpCode,0,1) eq "T") && ($t <= 0)){
		$herr = &Trim_Then_Append("Input temperature is zero","#");
		return;
	}
	if(($InpCode eq "TL") || ($InpCode eq "TLIQ") || ($InpCode eq "TVAP")){	#Calculate saturation values given temperature
		($t, $x[1], $phase, $p, $Dl, $Dv, $xliq[1], $xvap[1], $ierr, $herr) = (pack('d',$t),pack('d',$x[1]), pack('L!',$phase), pack('d',$p), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('L!',$ierr), pack('A255',$herr));
		$SATTdll->Call($t, $x[1], $phase, $p, $Dl, $Dv, $xliq[1], $xvap[1], $ierr, $herr, pack('L!',255));
		($t, $x[1], $phase, $p, $Dl, $Dv, $xliq[1], $xvap[1], $ierr, $herr) = (unpack('d',$t),unpack('d',$x[1]), unpack('L!',$phase), unpack('d',$p), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('L!',$ierr), unpack('A255',$herr));
		if((($p == 0) || ($Dl == 0)) && ($ierr == 0)){
			$ierr = 1;
			$herr = &Trim_Then_Append("Inputs are out of range","#");
			return;
		}
		($d,$q) = ($Dl,0);
		($d,$q) = ($Dv,1) if ($phase == 2);
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$p), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
		$THERMdll->Call($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$p), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
	}elsif(($InpCode eq "PL") || ($InpCode eq "PLIQ") || ($InpCode eq "PVAP")){	#Calculate saturation values given pressure
		($p, $x[1], $phase, $t, $Dl, $Dv, $xliq[1], $xvap[1], $ierr, $herr) = (pack('d',$p),pack('d',$x[1]), pack('L!',$phase), pack('d',$t), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('L!',$ierr), pack('A255',$herr));
		$SATPdll->Call($p, $x[1], $phase, $t, $Dl, $Dv, $xliq[1], $xvap[1], $ierr, $herr, pack('L!',255));
		($p, $x[1], $phase, $t, $Dl, $Dv, $xliq[1], $xvap[1], $ierr, $herr) = (unpack('d',$p),unpack('d',$x[1]), unpack('L!',$phase), unpack('d',$t), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('L!',$ierr), unpack('A255',$herr));
		if((($p == 0) || ($Dl == 0)) && ($ierr == 0)){
			$ierr = 1;
			$herr = &Trim_Then_Append("Inputs are out of range","#");
			return;
		}
		($d,$q) = ($Dl,0);
		($d,$q) = ($Dv,1) if ($phase == 2);
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$p), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
		$THERMdll->Call($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$p), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
	}elsif(($InpCode eq "DSAT") || ($InpCode eq "DL") || ($InpCode eq "DLIQ") || ($InpCode eq "DVAP")){	#Calculate saturation values given density
		($d, $x[1], $kr, $t, $p, $Dl, $Dv, $xliq[1], $xvap[1], $ierr, $herr) = (pack('d',$d),pack('d',$x[1]), pack('L!',$kr), pack('d',$t), pack('d',$p), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('L!',$ierr), pack('A255',$herr));
		$SATDdll->Call($d, $x[1], pack('L!',1), $kr, $t, $p, $Dl, $Dv, $xliq[1], $xvap[1], $ierr, $herr, pack('L!',255));
		($d, $x[1], $kr, $p, $t, $Dl, $Dv, $xliq[1], $xvap[1], $ierr, $herr) = (unpack('d',$d),unpack('d',$x[1]), unpack('L!',$kr), unpack('d',$t), unpack('d',$p), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('L!',$ierr), unpack('A255',$herr));
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$p), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
		$THERMdll->Call($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$p), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
		$q = $kr - 1;
	}elsif(($InpCode eq "HSAT") || ($InpCode eq "HSAT1")){	#Calculate properties given the enthalpy along the saturation line
		($h, $x[1],$nroot, $k1, $t, $p, $d, $k2, $t2, $p2, $d2, $ierr, $herr) = (pack('d',$h), pack('d',$x[1]),pack('L!',$nroot), pack('L!',$k1), pack('d',$t), pack('d',$p), pack('d',$d), pack('L!',$k2), pack('d',$t2), pack('d',$p2), pack('d',$d2), pack('L!',$ierr), pack('A255',$herr));
		$SATHdll->Call($h, $x[1], pack('L!',0), $nroot, $k1, $t, $p, $d, $k2, $t2, $p2, $d2, $ierr, $herr, pack('L!',255));
		($h, $x[1],$nroot, $k1, $t, $p, $d, $k2, $t2, $p2, $d2, $ierr, $herr) = (unpack('d',$h), unpack('d',$x[1]),unpack('L!',$nroot), unpack('L!',$k1), unpack('d',$t), unpack('d',$p), unpack('d',$d), unpack('L!',$k2), unpack('d',$t2), unpack('d',$p2), unpack('d',$d2), unpack('L!',$ierr), unpack('A255',$herr));
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$p), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
		$THERMdll->Call($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$p), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
		$q = $k1 - 1;
		if($nroot == 0){
			$ierr = 1;
			$herr = &Trim_Then_Append("Inputs are out of range","#");
			return;
		}
	}elsif($InpCode eq "HSAT2"){	#Calculate properties given the enthalpy along the saturation line for the second possible root
		($h, $x[1],$nroot, $k2, $t2, $p2, $d2, $k1, $t, $p, $d, $ierr, $herr) = (pack('d',$h), pack('d',$x[1]),pack('L!',$nroot), pack('L!',$k2), pack('d',$t2), pack('d',$p2), pack('d',$d2), pack('L!',$k1), pack('d',$t), pack('d',$p), pack('d',$d), pack('L!',$ierr), pack('A255',$herr));
		$SATHdll->Call($h, $x[1], pack('L!',0), $nroot, $k2, $t2, $p2, $d2, $k1, $t, $p, $d, $ierr, $herr, pack('L!',255));
		($h, $x[1],$nroot, $k2, $t2, $p2, $d2, $k1, $t, $p, $d, $ierr, $herr) = (unpack('d',$h), unpack('d',$x[1]),unpack('L!',$nroot), unpack('L!',$k2), unpack('d',$t2), unpack('d',$p2), unpack('d',$d2), unpack('L!',$k1), unpack('d',$t), unpack('d',$p), unpack('d',$d), unpack('L!',$ierr), unpack('A255',$herr));
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$p), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
		$THERMdll->Call($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$p), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
		$q = $k1 - 1;
		if($nroot < 2){
			$ierr = 1;
			$herr = &Trim_Then_Append("Inputs are out of range or there is no second root","#");
			return;
		}
	}elsif(($InpCode eq "SSAT") || ($InpCode eq "SSAT1")){	#Calculate properties given the entropy along the saturation line
		($s, $x[1],$nroot, $k1, $t, $p, $d, $k2, $t2, $p2, $d2,  $k3, $t3, $p3, $d3,$ierr, $herr) = (pack('d',$s), pack('d',$x[1]),pack('L!',$nroot), pack('L!',$k1), pack('d',$t), pack('d',$p), pack('d',$d), pack('L!',$k2), pack('d',$t2), pack('d',$p2), pack('d',$d2), pack('L!',$k3), pack('d',$t3), pack('d',$p3), pack('d',$d3),pack('L!',$ierr), pack('A255',$herr));
		$SATSdll->Call($s, $x[1], pack('L!',0), $nroot, $k1, $t, $p, $d, $k2, $t2, $p2, $d2, $ierr, $herr, pack('L!',255));
		($s, $x[1],$nroot, $k1, $t, $p, $d, $k2, $t2, $p2, $d2,  $k3, $t3, $p3, $d3,$ierr, $herr) = (unpack('d',$s), unpack('d',$x[1]),unpack('L!',$nroot), unpack('L!',$k1), unpack('d',$t), unpack('d',$p), unpack('d',$d), unpack('L!',$k2), unpack('d',$t2), unpack('d',$p2), unpack('d',$d2), unpack('L!',$k3), unpack('d',$t3), unpack('d',$p3), unpack('d',$d3),unpack('L!',$ierr), unpack('A255',$herr));
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$p), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
		$THERMdll->Call($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$p), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
		$q = $k1 - 1;
		if($nroot == 0){
			$ierr = 1;
			$herr = &Trim_Then_Append("Inputs are out of range","#");
			return;
		}
	}elsif($InpCode eq "SSAT2"){	#Calculate properties given the entropy along the saturation line for the second possible root
		($s, $x[1],$nroot, $k2, $t2, $p2, $d2, $k1, $t, $p, $d, $k3, $t3, $p3, $d3,$ierr, $herr) = (pack('d',$s), pack('d',$x[1]),pack('L!',$nroot), pack('L!',$k2), pack('d',$t2), pack('d',$p2), pack('d',$d2), pack('L!',$k1), pack('d',$t), pack('d',$p), pack('d',$d), pack('L!',$k3), pack('d',$t3), pack('d',$p3), pack('d',$d3), pack('L!',$ierr), pack('A255',$herr));
		$SATSdll->Call($s, $x[1], pack('L!',0), $nroot, $k2, $t2, $p2, $d2, $k1, $t, $p, $d, $ierr, $herr, pack('L!',255));
		($s, $x[1],$nroot, $k2, $t2, $p2, $d2, $k1, $t, $p, $d, $k3, $t3, $p3, $d3,$ierr, $herr) = (unpack('d',$s), unpack('d',$x[1]),unpack('L!',$nroot), unpack('L!',$k2), unpack('d',$t2), unpack('d',$p2), unpack('d',$d2), unpack('L!',$k1), unpack('d',$t), unpack('d',$p), unpack('d',$d), unpack('L!',$k3), unpack('d',$t3), unpack('d',$p3), unpack('d',$d3), unpack('L!',$ierr), unpack('A255',$herr));
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$p), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
		$THERMdll->Call($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$p), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
		$q = $k1 - 1;
		if($nroot < 2){
			$ierr = 1;
			$herr = &Trim_Then_Append("Inputs are out of range or there is no second root","#");
			return;
		}
	}elsif($InpCode eq "SSAT3"){	#Calculate properties given the entropy along the saturation line for the third possible root
		($s, $x[1],$nroot, $k3, $t3, $p3, $d3, $k2, $t2, $p2, $d2, $k1, $t, $p, $d, $ierr, $herr) = (pack('d',$s), pack('d',$x[1]),pack('L!',$nroot), pack('L!',$k3), pack('d',$t3), pack('d',$p3), pack('d',$d3), pack('L!',$k2), pack('d',$t2), pack('d',$p2), pack('d',$d2), pack('L!',$k1), pack('d',$t), pack('d',$p), pack('d',$d),  pack('L!',$ierr), pack('A255',$herr));
		$SATSdll->Call($s, $x[1], pack('L!',0), $nroot, $k2, $t2, $p2, $d2, $k1, $t, $p, $d, $ierr, $herr, pack('L!',255));
		($s, $x[1],$nroot, $k3, $t3, $p3, $d3, $k2, $t2, $p2, $d2, $k1, $t, $p, $d, $ierr, $herr) = (unpack('d',$s), unpack('d',$x[1]),unpack('L!',$nroot), unpack('L!',$k3), unpack('d',$t3), unpack('d',$p3), unpack('d',$d3), unpack('L!',$k2), unpack('d',$t2), unpack('d',$p2), unpack('d',$d2), unpack('L!',$k1), unpack('d',$t), unpack('d',$p), unpack('d',$d),  unpack('L!',$ierr), unpack('A255',$herr));
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$p), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
		$THERMdll->Call($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$p), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
		$q = $k1 - 1;
		if($nroot < 3){
			$ierr = 1;
			$herr = &Trim_Then_Append("Inputs are out of range or there is no third root","#");
			return;
		}
	}elsif(($InpCode eq "TPL") || ($InpCode eq "PTL")){
		($t, $p, $x[1], $d, $ierr, $herr) = (pack('d',$t), pack('d',$p), pack('d',$x[1]), pack('d',$d),  pack('L!',$ierr), pack('A255',$herr));
		$TPRHOdll->Call($t, $p, $x[1], pack('L!',1), pack('L!',0), $d, $ierr, $herr, pack('L!',255));
		($t, $p, $x[1], $d, $ierr, $herr) = (unpack('d',$t), unpack('d',$p), unpack('d',$x[1]), unpack('d',$d),  unpack('L!',$ierr), unpack('A255',$herr));
		($Dl,$Dv,$q) = ($d,$d,990);
		($t, $d, $x[1], $pp, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$pp), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
		$THERMdll->Call($t, $d, $x[1], $pp, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
		($t, $d, $x[1], $pp, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$pp), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
	}elsif(($InpCode eq "TPV") || ($InpCode eq "PTV")){
		($t, $p, $x[1], $d, $ierr, $herr) = (pack('d',$t), pack('d',$p), pack('d',$x[1]), pack('d',$d),  pack('L!',$ierr), pack('A255',$herr));
		$TPRHOdll->Call($t, $p, $x[1], pack('L!',2), pack('L!',0), $d, $ierr, $herr, pack('L!',255));
		($t, $p, $x[1], $d, $ierr, $herr) = (unpack('d',$t), unpack('d',$p), unpack('d',$x[1]), unpack('d',$d),  unpack('L!',$ierr), unpack('A255',$herr));
		($Dl,$Dv,$q) = ($d,$d,990);
		($t, $d, $x[1], $pp, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$pp), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
		$THERMdll->Call($t, $d, $x[1], $pp, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
		($t, $d, $x[1], $pp, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$pp), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
	}elsif(($InpCode eq "TP") || ($InpCode eq "PT")){
		($t, $p, $x[1], $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$t), pack('d',$p), pack('d',$x[1]), pack('d',$d), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
		$TPFLSHdll->Call($t, $p, $x[1], $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
		($t, $p, $x[1], $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$t), unpack('d',$p), unpack('d',$x[1]), unpack('d',$d), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
		if(($ierr == -16) && ($nc == 1)){#Point is below triple point, call SATT to see if a sublimation line is available and to check if the input p is less than the sublimation pressure.
			($t, $x[1], $pp, $Dl, $Dv, $xliq[1], $xvap[1], $ierr, $herr) = (pack('d',$t),pack('d',$x[1]), pack('d',$pp), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('L!',$ierr), pack('A255',$herr));
			$SATTdll->Call($t, $x[1], pack('L!',2), $pp, $Dl, $Dv, $xliq[1], $xvap[1], $ierr, $herr, pack('L!',255));
			($t, $x[1], $pp, $Dl, $Dv, $xliq[1], $xvap[1], $ierr, $herr) = (unpack('d',$t),unpack('d',$x[1]), unpack('d',$pp), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('L!',$ierr), unpack('A255',$herr));
			if(($p < $pp) && ($ierr <= 0)){
				($t, $p, $x[1], $d, $ierr, $herr) = (pack('d',$t), pack('d',$p), pack('d',$x[1]), pack('d',$d),  pack('L!',$ierr), pack('A255',$herr));
				$TPRHOdll->Call($t, $p, $x[1], pack('L!',2), pack('L!',0), $d, $ierr, $herr, pack('L!',255));
				($t, $p, $x[1], $d, $ierr, $herr) = (unpack('d',$t), unpack('d',$p), unpack('d',$x[1]), unpack('d',$d),  unpack('L!',$ierr), unpack('A255',$herr));
			}else{
				($ierr, $herr) = (-16, $herr2);
			}
			$q = 998;
		}
	}elsif(($InpCode eq "TD") || ($InpCode eq "DT")){
		($t, $d, $x[1], $p, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$p), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
		$TDFLSHdll->Call($t, $d, $x[1], $p, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
		($t, $d, $x[1], $p, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$p), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
	}elsif(($InpCode eq "TD&") || ($InpCode eq "DT&")){
		#Do not perform any flash calculation here
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$p), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
		$THERMdll->Call($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
		($t, $d, $x[1], $p, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$p), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
		$q = 990;
	}elsif((substr($InpCode,0,2) eq "TH") || (substr($InpCode,0,2) eq "HT")){
		if($InpCode =~ m/\</){
			($t, $h, $x[1], $p, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$t), pack('d',$h), pack('d',$x[1]), pack('d',$p), pack('d',$d), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$e), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
			$THFLSHdll->Call($t, $h, $x[1], pack('L!',1), $p,  $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
			($t, $h, $x[1], $p, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$t), unpack('d',$h), unpack('d',$x[1]), unpack('d',$p), unpack('d',$d), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$e), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
		}else{
			($t, $h, $x[1], $p, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$t), pack('d',$h), pack('d',$x[1]), pack('d',$p), pack('d',$d), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$e), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
			$THFLSHdll->Call($t, $h, $x[1], pack('L!',2), $p,  $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
			($t, $h, $x[1], $p, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$t), unpack('d',$h), unpack('d',$x[1]), unpack('d',$p), unpack('d',$d), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$e), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
		}
	}elsif(($InpCode eq "TS") || ($InpCode eq "ST")){
		($t, $s, $x[1], $p, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$t), pack('d',$s), pack('d',$x[1]), pack('d',$p), pack('d',$d), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$e), pack('d',$h), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
		$TSFLSHdll->Call($t, $s, $x[1], pack('L!',1), $p,  $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
		($t, $s, $x[1], $p, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$t), unpack('d',$s), unpack('d',$x[1]), unpack('d',$p), unpack('d',$d), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$e), unpack('d',$h), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
	}elsif(($InpCode eq "TE") || ($InpCode eq "ET")){
		($t, $e, $x[1], $p, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$t), pack('d',$e), pack('d',$x[1]), pack('d',$p), pack('d',$d), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
		$TEFLSHdll->Call($t, $e, $x[1], pack('L!',2), $p,  $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
		($t, $e, $x[1], $p, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$t), unpack('d',$e), unpack('d',$x[1]), unpack('d',$p), unpack('d',$d), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
	}elsif(($InpCode eq "TQ") || ($InpCode eq "QT")){
		($t, $q, $x[1], $molmass, $p, $d, $Dl, $Dv, $xliq[1], $xvap[1], $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$t), pack('d',$q), pack('d',$x[1]), pack('L!',$molmass), pack('d',$p), pack('d',$d), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
		$TQFLSHdll->Call($t, $q, $x[1], $molmass, $p,  $d, $Dl, $Dv, $xliq[1], $xvap[1], $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
		($t, $q, $x[1], $molmass, $p, $d, $Dl, $Dv, $xliq[1], $xvap[1], $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$t), unpack('d',$q), unpack('d',$x[1]), unpack('L!',$molmass), unpack('d',$p), unpack('d',$d), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
		if($molmass == 2){
			($xliq[1], $xlkg[1], $wmix) = (pack('d',$xliq[1]), pack('d',$xlkg[1]), pack('d',$wmix));
			$XMASSdll->($xliq[1], $xlkg[1], $wmix);
			($xliq[1], $xlkg[1], $wmix) = (unpack('d',$xliq[1]), unpack('d',$xlkg[1]), unpack('d',$wmix));
			($xvap[1], $xvkg[1], $wmix) = (pack('d',$xvap[1]), pack('d',$xvkg[1]), pack('d',$wmix));
			$XMASSdll->($xvap[1], $xvkg[1], $wmix);
			($xvap[1], $xvkg[1], $wmix) = (unpack('d',$xvap[1]), unpack('d',$xvkg[1]), unpack('d',$wmix));
			($q, $xlkg[1], $xvkg[1], $qmol, $xlj[1], $xvj[1], $wliq, $wvap, $ierr, $herr) = (pack('d',$q), pack('d',$xlkg[1]), pack('d',$xvkg[1]), pack('d',$qmol), pack('d',$xlj[1]), pack('d',$xvj[1]), pack('d',$wliq), pack('d',$wvap), pack('L!',$ierr), pack('A255',$herr));
			$QMOLEdll->($q, $xlkg[1], $xvkg[1], $qmol, $xlj[1], $xvj[1], $wliq, $wvap, $ierr, $herr, pack('L!',255));
			($q, $xlkg[1], $xvkg[1], $qmol, $xlj[1], $xvj[1], $wliq, $wvap, $ierr, $herr) = (unpack('d',$q), unpack('d',$xlkg[1]), unpack('d',$xvkg[1]), unpack('d',$qmol), unpack('d',$xlj[1]), unpack('d',$xvj[1]), unpack('d',$wliq), unpack('d',$wvap), unpack('L!',$ierr), unpack('A255',$herr));
	      	$q = $qmol;
		}
	}elsif(($InpCode eq "PD") || ($InpCode eq "DP")){
		($p, $d, $x[1], $t, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$p), pack('d',$d), pack('d',$x[1]), pack('d',$t), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
		$PDFLSHdll->Call($p, $d, $x[1], $t, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
		($p, $d, $x[1], $t, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$p), unpack('d',$d), unpack('d',$x[1]), unpack('d',$t), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
	}elsif(($InpCode eq "PH") || ($InpCode eq "HP")){
		($p, $h, $x[1], $t, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$p), pack('d',$h), pack('d',$x[1]), pack('d',$t), pack('d',$d), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$e), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
		$PHFLSHdll->Call($p, $h, $x[1], $t,  $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
		($p, $h, $x[1], $t, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$p), unpack('d',$h), unpack('d',$x[1]), unpack('d',$t), unpack('d',$d), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$e), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
	}elsif(($InpCode eq "PS") || ($InpCode eq "SP")){
		($p, $s, $x[1], $t, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$p), pack('d',$s), pack('d',$x[1]), pack('d',$t), pack('d',$d), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$e), pack('d',$h), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
		$PSFLSHdll->Call($p, $s, $x[1], $t, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
		($p, $s, $x[1], $t, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$p), unpack('d',$s), unpack('d',$x[1]), unpack('d',$t), unpack('d',$d), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$e), unpack('d',$h), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
	}elsif(($InpCode eq "PE") || ($InpCode eq "EP")){
		($p, $e, $x[1], $t, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$p), pack('d',$e), pack('d',$x[1]), pack('d',$t), pack('d',$d), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
		$PEFLSHdll->Call($p, $e, $x[1], $t,  $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
		($p, $e, $x[1], $t, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$p), unpack('d',$e), unpack('d',$x[1]), unpack('d',$t), unpack('d',$d), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
	}elsif(($InpCode eq "PQ") || ($InpCode eq "QP")){
		($p, $q, $x[1], $molmass, $t, $d, $Dl, $Dv, $xliq[1], $xvap[1], $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$p), pack('d',$q), pack('d',$x[1]), pack('L!',$molmass), pack('d',$t), pack('d',$d), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
		$PQFLSHdll->Call($p, $q, $x[1], $molmass, $t,  $d, $Dl, $Dv, $xliq[1], $xvap[1], $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
		($p, $q, $x[1], $molmass, $t, $d, $Dl, $Dv, $xliq[1], $xvap[1], $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$p), unpack('d',$q), unpack('d',$x[1]), unpack('L!',$molmass), unpack('d',$t), unpack('d',$d), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
		if($molmass == 2){
			($xliq[1], $xlkg[1], $wmix) = (pack('d',$xliq[1]), pack('d',$xlkg[1]), pack('d',$wmix));
			$XMASSdll->($xliq[1], $xlkg[1], $wmix);
			($xliq[1], $xlkg[1], $wmix) = (unpack('d',$xliq[1]), unpack('d',$xlkg[1]), unpack('d',$wmix));
			($xvap[1], $xvkg[1], $wmix) = (pack('d',$xvap[1]), pack('d',$xvkg[1]), pack('d',$wmix));
			$XMASSdll->($xvap[1], $xvkg[1], $wmix);
			($xvap[1], $xvkg[1], $wmix) = (unpack('d',$xvap[1]), unpack('d',$xvkg[1]), unpack('d',$wmix));
			($q, $xlkg[1], $xvkg[1], $qmol, $xlj[1], $xvj[1], $wliq, $wvap, $ierr, $herr) = (pack('d',$q), pack('d',$xlkg[1]), pack('d',$xvkg[1]), pack('d',$qmol), pack('d',$xlj[1]), pack('d',$xvj[1]), pack('d',$wliq), pack('d',$wvap), pack('L!',$ierr), pack('A255',$herr));
			$QMOLEdll->($q, $xlkg[1], $xvkg[1], $qmol, $xlj[1], $xvj[1], $wliq, $wvap, $ierr, $herr, pack('L!',255));
			($q, $xlkg[1], $xvkg[1], $qmol, $xlj[1], $xvj[1], $wliq, $wvap, $ierr, $herr) = (unpack('d',$q), unpack('d',$xlkg[1]), unpack('d',$xvkg[1]), unpack('d',$qmol), unpack('d',$xlj[1]), unpack('d',$xvj[1]), unpack('d',$wliq), unpack('d',$wvap), unpack('L!',$ierr), unpack('A255',$herr));
			$q = $qmol;
		}
	}elsif(($InpCode eq "DH") || ($InpCode eq "HD")){
		($d, $h, $x[1], $t, $p, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$d), pack('d',$h), pack('d',$x[1]), pack('d',$t), pack('d',$p), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$e), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
		$DHFLSHdll->Call($d, $h, $x[1], $t,  $p, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
		($d, $h, $x[1], $t, $p, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$d), unpack('d',$h), unpack('d',$x[1]), unpack('d',$t), unpack('d',$p), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$e), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
	}elsif(($InpCode eq "DS") || ($InpCode eq "SD")){
		($d, $s, $x[1], $t, $p, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$d), pack('d',$s), pack('d',$x[1]), pack('d',$t), pack('d',$p), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$e), pack('d',$h), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
		$DSFLSHdll->Call($d, $s, $x[1], $t, $p, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
		($d, $s, $x[1], $t, $p, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$d), unpack('d',$s), unpack('d',$x[1]), unpack('d',$t), unpack('d',$p), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$e), unpack('d',$h), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
	}elsif(($InpCode eq "DE") || ($InpCode eq "ED")){
		($d, $e, $x[1], $t, $p, $Dl, $Dv, $xliq[1], $xvap[1], $q, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$d), pack('d',$e), pack('d',$x[1]), pack('d',$t), pack('d',$p), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
		$DEFLSHdll->Call($d, $e, $x[1], $t,  $p, $Dl, $Dv, $xliq[1], $xvap[1], $q, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
		($d, $e, $x[1], $t, $p, $Dl, $Dv, $xliq[1], $xvap[1], $q, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$d), unpack('d',$e), unpack('d',$x[1]), unpack('d',$t), unpack('d',$p), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
	}elsif(($InpCode eq "HS") || ($InpCode eq "SH")){
		($h, $s, $x[1], $t, $p, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$h), pack('d',$s), pack('d',$x[1]), pack('d',$t), pack('d',$p), pack('d',$d), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$e), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
		$HSFLSHdll->Call($h, $s, $x[1], $t, $p, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
		($h, $s, $x[1], $t, $p, $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$h), unpack('d',$s), unpack('d',$x[1]), unpack('d',$t), unpack('d',$p), unpack('d',$d), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$e), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
	}elsif($InpCode eq "TMELT"){
		($t, $x[1], $p, $ierr, $herr) = (pack('d',$t), pack('d',$x[1]), pack('d',$p), pack('L!',$ierr), pack('A255',$herr));
		$MELTTdll->Call($t, $x[1], $p, $ierr, $herr, pack('L!',255));
		($t, $x[1], $p, $ierr, $herr) = (unpack('d',$t), unpack('d',$x[1]), unpack('d',$p), unpack('L!',$ierr), unpack('A255',$herr));
		if($ierr == 0){
			($t, $p, $x[1], $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$t), pack('d',$p), pack('d',$x[1]), pack('d',$d), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
			$TPFLSHdll->Call($t, $p, $x[1], $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
			($t, $p, $x[1], $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$t), unpack('d',$p), unpack('d',$x[1]), unpack('d',$d), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
		}
	}elsif($InpCode eq "PMELT"){
		if($p == 0){
			$ierr = 1;
			$herr = &Trim_Then_Append("Input pressure is zero","#");
			return;
		}
		($p, $x[1], $t, $ierr, $herr) = (pack('d',$p), pack('d',$x[1]), pack('d',$t), pack('L!',$ierr), pack('A255',$herr));
		$MELTPdll->Call($p, $x[1], $t, $ierr, $herr, pack('L!',255));
		($p, $x[1], $t, $ierr, $herr) = (unpack('d',$p), unpack('d',$x[1]), unpack('d',$t), unpack('L!',$ierr), unpack('A255',$herr));
		if($ierr == 0){
			($t, $p, $x[1], $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (pack('d',$t), pack('d',$p), pack('d',$x[1]), pack('d',$d), pack('d',$Dl), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('d',$q), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('L!',$ierr), pack('A255',$herr));
			$TPFLSHdll->Call($t, $p, $x[1], $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr, pack('L!',255));
			($t, $p, $x[1], $d, $Dl, $Dv, $xliq[1], $xvap[1], $q, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $ierr, $herr) = (unpack('d',$t), unpack('d',$p), unpack('d',$x[1]), unpack('d',$d), unpack('d',$Dl), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('d',$q), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('L!',$ierr), unpack('A255',$herr));
		}
	}elsif($InpCode eq "TSUBL"){
		($t, $x[1], $p, $ierr, $herr) = (pack('d',$t), pack('d',$x[1]), pack('d',$p), pack('L!',$ierr), pack('A255',$herr));
		$SUBLTdll->Call($t, $x[1], $p, $ierr, $herr, pack('L!',255));
		($t, $x[1], $p, $ierr, $herr) = (unpack('d',$t), unpack('d',$x[1]), unpack('d',$p), unpack('L!',$ierr), unpack('A255',$herr));
		if(($ierr == 0) && ($p == 0)){
			$ierr = 1;
			$herr = &Trim_Then_Append("No sublimation line available","#");
		}
		if($ierr == 0){
			$q = 1;
			$d = $p / 8.314472 / $t;
			($t, $p, $x[1], $d, $ierr, $herr) = (pack('d',$t), pack('d',$p), pack('d',$x[1]), pack('d',$d),  pack('L!',$ierr), pack('A255',$herr));
			$TPRHOdll->Call($t, $p, $x[1], pack('L!',2), pack('L!',1), $d, $ierr, $herr, pack('L!',255));
			($t, $p, $x[1], $d, $ierr, $herr) = (unpack('d',$t), unpack('d',$p), unpack('d',$x[1]), unpack('d',$d),  unpack('L!',$ierr), unpack('A255',$herr));
			($t, $d, $x[1], $pp, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$pp), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
			$THERMdll->Call($t, $d, $x[1], $pp, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
			($t, $d, $x[1], $pp, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$pp), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
		}
	}elsif($InpCode eq "PSUBL"){
		if($p == 0){
			$ierr = 1;
			$herr = &Trim_Then_Append("Input pressure is zero","#");
			return;
		}
		($p, $x[1], $t, $ierr, $herr) = (pack('d',$p), pack('d',$x[1]), pack('d',$t), pack('L!',$ierr), pack('A255',$herr));
		$SUBLPdll->Call($p, $x[1], $t, $ierr, $herr, pack('L!',255));
		($p, $x[1], $t, $ierr, $herr) = (unpack('d',$p), unpack('d',$x[1]), unpack('d',$t), unpack('L!',$ierr), unpack('A255',$herr));
		if(($ierr == 0) && ($t == 0)){
			$ierr = 1;
			$herr = &Trim_Then_Append("No sublimation line available","#");
		}
		if($ierr == 0){
			$q = 1;
			$d = $p / 8.314472 / $t;
			($t, $p, $x[1], $d, $ierr, $herr) = (pack('d',$t), pack('d',$p), pack('d',$x[1]), pack('d',$d),  pack('L!',$ierr), pack('A255',$herr));
			$TPRHOdll->Call($t, $p, $x[1], pack('L!',2), pack('L!',1), $d, $ierr, $herr, pack('L!',255));
			($t, $p, $x[1], $d, $ierr, $herr) = (unpack('d',$t), unpack('d',$p), unpack('d',$x[1]), unpack('d',$d),  unpack('L!',$ierr), unpack('A255',$herr));
			($t, $d, $x[1], $pp, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$pp), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
			$THERMdll->Call($t, $d, $x[1], $pp, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
			($t, $d, $x[1], $pp, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$pp), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
		}
	}else{
		$ierr = 1;
		$herr = &Trim_Then_Append("Invalid input code","#");
	}
	return;
}

sub CalcProp{	#(FluidName, InpCode, ByVal Units, ByVal Prop1, ByVal Prop2)
	my ($FluidName, $InpCode, $Units, $Prop1, $Prop2) = @_;
	my ($iflag1, $iflag2);	#VBA As Integer
	our ($CRITPdll,$THERMdll,$MAXTdll,$MAXPdll,$INFOdll,$SATTdll);
	($q, $t, $p, $d, $Dl, $Dv, $e, $h, $s, $Cvcalc, $Cpcalc, $w)=(0,0,0,0,0,0,0,0,0,0,0,0);
	$iflag1 = 1 if ($Prop1 == undef);
	unless($iflag1){
		$iflag1 = 2 unless(length(&Trim_Then_Append($Prop1)));
		unless($iflag1){
			if(($Prop1 == 0) && ($Prop1 ne "0")){
				$ierr = 1;
				$herr = &Trim_Then_Append("Invalid input: ","#") . $Prop1;
				return;
			}
		}
	}
	$iflag2 = 1 if ($Prop2 == undef);
	unless($iflag2){
		$iflag2 = 2 unless(length(&Trim_Then_Append($Prop2)));
		unless($iflag2){
			if(($Prop2 == 0) && ($Prop2 ne "0")){
				$ierr = 1;
				$herr = &Trim_Then_Append("Invalid input: ","#") . $Prop2;
				return;
			}
		}
	}
	$InpCode = "" if ($InpCode eq undef);
	($FluidName, $InpCode, $Units, $Prop1, $Prop2) = &CalcSetup($FluidName, $InpCode, $Units, $Prop1, $Prop2);
	if(uc(substr($InpCode,0,4)) eq "CRIT"){
		($x[1],$t,$p,$d,$ierr,$herr) = (pack('d',$x[1]),pack('d',$t),pack('d',$p),pack('d',$d),pack('L!',$ierr),pack('A255',$herr));
		$CRITPdll->Call($x[1],$t,$p,$d,$ierr,$herr,pack('L!',255));
		($x[1],$t,$p,$d,$ierr,$herr) = (unpack('d',$x[1]),unpack('d',$t),unpack('d',$p),unpack('d',$d),unpack('L!',$ierr),unpack('A255',$herr));
		unless($ierr){
			($t, $d, $x[1], $pc, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$pc), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
			$THERMdll->Call($t, $d, $x[1], $pc, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
			($t, $d, $x[1], $pc, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$pc), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
		}
		return;
	}elsif(uc(substr($InpCode,0,7)) eq "SATMAXT"){
		($x[1],$t,$p,$d,$ierr,$herr) = (pack('d',$x[1]),pack('d',$t),pack('d',$p),pack('d',$d),pack('L!',$ierr),pack('A255',$herr));
		$MAXTdll->Call($x[1],$t,$p,$d,$ierr,$herr,pack('L!',255));
		($x[1],$t,$p,$d,$ierr,$herr) = (unpack('d',$x[1]),unpack('d',$t),unpack('d',$p),unpack('d',$d),unpack('L!',$ierr),unpack('A255',$herr));
		unless($ierr){
			($t, $d, $x[1], $pc, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$pc), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
			$THERMdll->Call($t, $d, $x[1], $pc, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
			($t, $d, $x[1], $pc, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$pc), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
		}
		return;
	}elsif(uc(substr($InpCode,0,7)) eq "SATMAXP"){
		($x[1],$t,$p,$d,$ierr,$herr) = (pack('d',$x[1]),pack('d',$t),pack('d',$p),pack('d',$d),pack('L!',$ierr),pack('A255',$herr));
		$MAXPdll->Call($x[1],$t,$p,$d,$ierr,$herr,pack('L!',255));
		($x[1],$t,$p,$d,$ierr,$herr) = (unpack('d',$x[1]),unpack('d',$t),unpack('d',$p),unpack('d',$d),unpack('L!',$ierr),unpack('A255',$herr));
		unless($ierr){
			($t, $d, $x[1], $pc, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$pc), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
			$THERMdll->Call($t, $d, $x[1], $pc, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
			($t, $d, $x[1], $pc, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$pc), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
		}
		return;
	}elsif(uc(substr($InpCode,0,4)) eq "TRIP"){
		if($nc != 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("Can only return triple point for a pure fluid","#");
			return;
		}
		($wmm, $t, $tnbpt, $tc, $pc, $dc, $Zc, $acf, $dip, $Rgas) = (pack('d',$wmm), pack('d',$t), pack('d',$tnbpt), pack('d',$tc), pack('d',$pc), pack('d',$dc), pack('d',$Zc), pack('d',$acf), pack('d',$dip), pack('d',$Rgas));
		$INFOdll->Call(pack('L!',1), $wmm, $t, $tnbpt, $tc, $pc, $dc, $Zc, $acf, $dip, $Rgas);
		($wmm, $t, $tnbpt, $tc, $pc, $dc, $Zc, $acf, $dip, $Rgas) = (unpack('d',$wmm), unpack('d',$t), unpack('d',$tnbpt), unpack('d',$tc), unpack('d',$pc), unpack('d',$dc), unpack('d',$Zc), unpack('d',$acf), unpack('d',$dip), unpack('d',$Rgas));
		($t, $x[1], $p, $d, $Dv, $xliq[1], $xvap[1], $ierr, $herr) = (pack('d',$t), pack('d',$x[1]), pack('d',$p), pack('d',$d), pack('d',$Dv), pack('d',$xliq[1]), pack('d',$xvap[1]), pack('L!',$ierr), pack('A255',$herr));
		$SATTdll->Call($t, $x[1], pack('L!',1), $p, $d, $Dv, $xliq[1], $xvap[1], $ierr, $herr, pack('L!',255));
		($t, $x[1], $p, $d, $Dv, $xliq[1], $xvap[1], $ierr, $herr) = (unpack('d',$t), unpack('d',$x[1]), unpack('d',$p), unpack('d',$d), unpack('d',$Dv), unpack('d',$xliq[1]), unpack('d',$xvap[1]), unpack('L!',$ierr), unpack('A255',$herr));
		unless($ierr){
			($t, $d, $x[1], $pc, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (pack('d',$t), pack('d',$d), pack('d',$x[1]), pack('d',$pc), pack('d',$e), pack('d',$h), pack('d',$s), pack('d',$Cvcalc), pack('d',$Cpcalc), pack('d',$w), pack('d',$hjt));
			$THERMdll->Call($t, $d, $x[1], $pc, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt);
			($t, $d, $x[1], $pc, $e, $h, $s, $Cvcalc, $Cpcalc, $w, $hjt) = (unpack('d',$t), unpack('d',$d), unpack('d',$x[1]), unpack('d',$pc), unpack('d',$e), unpack('d',$h), unpack('d',$s), unpack('d',$Cvcalc), unpack('d',$Cpcalc), unpack('d',$w), unpack('d',$hjt));
		}
		return;
	}
	if($iflag1){
		$ierr = 1;
		$herr = &Trim_Then_Append("Inputs are missing","#");
		return;
	}
	if($ierr > 0){
		return;
	}
	&Calc($InpCode, $Prop1, $Prop2, $iflag1, $iflag2) if $InpCode ne "";
}

sub CalcSetup{	#(FluidName, InpCode, Units, Prop1, Prop2)
	my ($FluidName, $InpCode, $Units, $Prop1, $Prop2) = @_;
	
	if(&Trim_Then_Append($FluidName) eq ""){
		$herr = &Trim_Then_Append("Invalid inputs","#");
	}else{
		$FluidName = &Setup($FluidName);
		unless($ierr > 0){
			($InpCode, $Units, $Prop1, $Prop2) = &ConvertUnits($InpCode, $Units, $Prop1, $Prop2);
			$herr = "";
			($q, $t, $p, $d, $Dl, $Dv, $e, $h, $s, $Cvcalc, $Cpcalc, $w)=(0,0,0,0,0,0,0,0,0,0,0,0);
		}
	}
	return $FluidName, $InpCode, $Units, $Prop1, $Prop2;
}

sub CheckName{	#(FluidName)
	my $FluidName = shift;
	$FluidName =~ s/^"+//;	#removes all leading double quotes
	$FluidName =~ s/"+$//;	#removes all trailing double quotes
	
	$FluidName = "nitrogen;7812;argon;0092;oxygen;2096" if uc($FluidName) eq "AIR";
	$FluidName = "CO2" if uc($FluidName) eq "CARBON DIOXIDE";
	$FluidName = "CO" if uc($FluidName) eq "CARBON MONOXIDE";
	$FluidName = "COS" if uc($FluidName) eq "CARBONYL SULFIDE";
	$FluidName = "CYCLOHEX" if uc($FluidName) eq "CYCLOHEXANE";
	$FluidName = "CYCLOPRO" if uc($FluidName) eq "CYCLOPROPANE";
	$FluidName = "D2" if uc($FluidName) eq "DEUTERIUM";
	$FluidName = "D2O" if uc($FluidName) eq "HEAVY WATER";
	$FluidName = "H2S" if uc($FluidName) eq "HYDROGEN SULFIDE";
	$FluidName = "ISOBUTAN" if uc($FluidName) eq "IBUTANE";
	$FluidName = "ISOBUTAN" if uc($FluidName) eq "ISOBUTANE";
	$FluidName = "IPENTANE" if uc($FluidName) eq "ISOPENTANE";
	$FluidName = "NEOPENTN" if uc($FluidName) eq "NEOPENTANE";
	$FluidName = "IHEXANE" if uc($FluidName) eq "ISOHEXANE";
	$FluidName = "N2O" if uc($FluidName) eq "NITROUS OXIDE";
	$FluidName = "PARAHYD" if uc($FluidName) eq "PARAHYDROGEN";
	$FluidName = "PROPYLEN" if uc($FluidName) eq "PROPYLENE";
	$FluidName = "SF6" if uc($FluidName) eq "SULFUR HEXAFLUORIDE";
	$FluidName = "DME" if uc($FluidName) eq "DIMETHYLETHER";
	$FluidName = "1BUTENE" if uc($FluidName) eq "BUTENE";
	$FluidName = "IBUTENE" if uc($FluidName) eq "ISOBUTENE";
	$FluidName = "C2BUTENE" if uc($FluidName) eq "CIS-BUTENE";
	$FluidName = "T2BUTENE" if uc($FluidName) eq "TRANS-BUTENE";
	$FluidName = "CF3I" if uc($FluidName) eq "TRIFLUOROIODOMETHANE";
	$FluidName = "C4F10" if uc($FluidName) eq "PERFLUOROBUTANE";
	$FluidName = "C5F12" if uc($FluidName) eq "PERFLUOROPENTANE";
	$FluidName = "NF3" if uc($FluidName) eq "NITROGEN TRIFLUORIDE";
	$FluidName = "SO2" if uc($FluidName) eq "SULFUR DIOXIDE";
	$FluidName = "SF6" if uc($FluidName) eq "SULFUR HEXAFLUORIDE";
	$FluidName = "C12" if uc($FluidName) eq "DODECANE";
	$FluidName = "DME" if uc($FluidName) eq "DIMETHYLETHER";
}

sub ConvertUnits{	#(InpCode, Units, Prop1, Prop2)
	my ($InpCode, $Units, $Prop1, $Prop2) = @_;
	my ($i, $DefaultUnits);	#VBA As Integer
	my ($at, $bt);	#VBA As String
	my $tConv;	#VBA As Double
	my $return_value;
	
	$InpCode = $InpCode || "";
	$Units = $Units || "";
	$Prop1 = $Prop1 || 0;
	$Prop2 = $Prop2 || 0;
	return &Trim_Then_Append($herr,"#") if (($ierr > 0) || ($ierr == -16));
	
	&SetupUnits(0) if $tUnits2 eq "";	#If Default units are changed, this needs to be called again.  Normally it is skipped after the first entry
	$DefaultUnits = 0;	#Change the 0 in the following line to 3 for default English units, 1 for mks, or 2 for cgs, etc.
	$i = $DefaultUnits;
	
	#Do not change the order of the next block of statements
	$i = 0 if (substr(uc($Units),0,2) eq "SI");								#SI
	$i = 5 if ((uc($Units) eq "SI WITH C") || (uc($Units) eq "SIWITHC"));	#SI with C
	$i = 5 if (uc($Units) eq "C");											#SI with C
	$i = 4 if (substr(uc($Units),0,1) eq "M");								#Mixed
	$i = 6 if (uc($Units) eq "MOLAR SI");									#Molar SI
	$i = 6 if (uc($Units) eq "MOLARSI");									#Molar SI
	$i = 1 if (uc($Units) eq "MKS");										#mks
	$i = 2 if (uc($Units) eq "CGS");										#cgs
	$i = 3 if (substr(uc($Units),0,1) eq "E");								#English
	$i = 7 if (uc($Units) eq "MOLAR E");									#Molar English
	$i = 9 if (uc($Units) eq "MEUNITS");									#Mechanical Engineering units

	$at = uc(substr($InpCode,0,1));
	$bt = uc(substr($InpCode,1,1));
	$molmass = 1;
	$molmass = 2 unless (lc($dUnits[$i])=~ m/(mol)/);
	if($at eq "-"){
		$return_value = $Prop1;
		return &Trim_Then_Append("Undefined","#") if (($Prop1 == -9999999) || ($Prop1 == -9999900) || (($Prop1 > -9999999) && ($Prop1 < -9999900)));
		$return_value = &UnitConvert($Prop1, "T", $tUnits2, $tUnits[$i]) if ($bt eq "T");
		$return_value = &UnitConvert($Prop1, "T", $taUnits2, $taUnits[$i]) if ($bt eq "A");
		$return_value = &UnitConvert($Prop1, "P", $pUnits2, $pUnits[$i]) if ($bt eq "P");
		$return_value = &UnitConvert($Prop1, "D", $dUnits2, $dUnits[$i]) if ($bt eq "D");
		$return_value = &UnitConvert($Prop1, "D", $vUnits2, $vUnits[$i]) if ($bt eq "V");
		$return_value = &UnitConvert($Prop1, "H", $hUnits2, $hUnits[$i]) if (($bt eq "H") || ($bt eq "E"));
		$return_value = &UnitConvert($Prop1, "S", $sUnits2, $sUnits[$i]) if ($bt eq "S");
		$return_value = &UnitConvert($Prop1, "W", $wUnits2, $wUnits[$i]) if ($bt eq "W");
		$return_value = &UnitConvert($Prop1, "U", $visUnits2, $visUnits[$i]) if ($bt eq "U");
		$return_value = &UnitConvert($Prop1, "K", $tcxUnits2, $tcxUnits[$i]) if ($bt eq "K");
		$return_value = &UnitConvert($Prop1, "N", $stUnits2, $stUnits[$i]) if ($bt eq "N");
		if ($bt eq "J"){
			$tConv = 1;
			$tConv = 1 / $RtoK if (($tUnits[$i] eq "R") || ($tUnits[$i] eq "F")); 
			$return_value = $Prop1 * $tConv / &UnitConvert(1, "P", "kPa", $pUnits[$i]);
		}
		if ($bt eq "I"){
		#convert cm^2/s to ft^2/s
			$return_value = $Prop1 * &UnitConvert(1, "A", "cm^2", "ft^2") if (($i == 3) || ($i == 7));
		}
		return $return_value;
	}else{
		if(&Trim_Then_Append($Prop1)){
			$Prop1 = &UnitConvert($Prop1, "T", $tUnits[$i], $tUnits2) if ($at eq "T");
			$Prop1 = &UnitConvert($Prop1, "T", $taUnits[$i], $taUnits2) if ($at eq "A");
			$Prop1 = &UnitConvert($Prop1, "P", $pUnits[$i], $pUnits2) if ($at eq "P");
			$Prop1 = &UnitConvert($Prop1, "D", $dUnits[$i], $dUnits2) if ($at eq "D");
			$Prop1 = &UnitConvert($Prop1, "D", $vUnits[$i], $vUnits2) if ($at eq "V");
			$Prop1 = &UnitConvert($Prop1, "H", $hUnits[$i], $hUnits2) if (($at eq "H") || ($at eq "E"));
			$Prop1 = &UnitConvert($Prop1, "S", $sUnits[$i], $sUnits2) if ($at eq "S");
			$Prop1 = &UnitConvert($Prop1, "W", $wUnits[$i], $wUnits2) if ($at eq "W");
			$Prop1 = &UnitConvert($Prop1, "U", $visUnits[$i], $visUnits2) if ($at eq "U");
			$Prop1 = &UnitConvert($Prop1, "K", $tcxUnits[$i], $tcxUnits2) if ($at eq "K");
			$Prop1 = &UnitConvert($Prop1, "N", $stUnits[$i], $stUnits2) if ($at eq "N");
		}	
		if(&Trim_Then_Append($Prop2)){
			$Prop2 = &UnitConvert($Prop2, "T", $tUnits[$i], $tUnits2) if ($bt eq "T");
			$Prop2 = &UnitConvert($Prop2, "T", $taUnits[$i], $taUnits2) if ($bt eq "A");
			$Prop2 = &UnitConvert($Prop2, "P", $pUnits[$i], $pUnits2) if ($bt eq "P");
			$Prop2 = &UnitConvert($Prop2, "D", $dUnits[$i], $dUnits2) if ($bt eq "D");
			$Prop2 = &UnitConvert($Prop2, "D", $vUnits[$i], $vUnits2) if ($bt eq "V");
			$Prop2 = &UnitConvert($Prop2, "H", $hUnits[$i], $hUnits2) if (($bt eq "H") || ($bt eq "E"));
			$Prop2 = &UnitConvert($Prop2, "S", $sUnits[$i], $sUnits2) if ($bt eq "S");
			$Prop2 = &UnitConvert($Prop2, "W", $wUnits[$i], $wUnits2) if ($bt eq "W");
			$Prop2 = &UnitConvert($Prop2, "U", $visUnits[$i], $visUnits2) if ($bt eq "U");
			$Prop2 = &UnitConvert($Prop2, "K", $tcxUnits[$i], $tcxUnits2) if ($bt eq "K");
			$Prop2 = &UnitConvert($Prop2, "N", $stUnits[$i], $stUnits2) if ($bt eq "N");
		}
		return $InpCode, $Units, $Prop1, $Prop2;
	}	
}

sub Setup {	#Takes a fluid's name as the only argument	
	my $FluidName = $_[0];
	my ($a, $ab, $FluidNme, $FlNme, $htype, $hmix, $hcomp);	#VBA As String
	my ($i, $sc, $ncc, $Mass);	#VBA As Integer
	my ($sum, $hRef, $sRef, $Tref, $pref);	#VBA As Double
	my ($nc2);	#VBA As Long
	my (@xtemp);	#VBA (1 To MaxComps) As Double
	my (@split_string,$split_element, $return_value);
	our ($FluidsDirectory,$MixturesDirectory,$SETMIXdll,$SETUPdll,$SETREFdll,$GERG04dll,$SETMODdll,$PREOSdll,$XMOLEdll);
		
	$FlNme = $FluidName;
	($htype, $hmix, $hcomp) = ( " " x 3,  " " x 3,  " " x 60);
	$ierr = 0;
	$herr = "";
	
	if( ( ( ($FluidName =~ m/(error)/) || ($FluidName =~ m/(Inputs are out of range)/) ) || ($FluidName eq $FldOld) ) ){return $FluidName;}
	
	$FldOld = "";
	&CheckName(\$FluidName);
	$hrf = "DEF";
	$hfmix = $FluidsDirectory . "hmx.bnc";
	$a = "";
	$Mass = 0;

	if(uc($FluidName) =~ m/\.(MIX)/){	#Set up predefined mixtures (using *.MIX)
		$hmxnme = $MixturesDirectory . $FluidName;
		
		($hmxnme, $hfmix, $hrf, $nc2, $hfld, $xtemp[1], $ierr, $herr) = (pack('A255',$hfmix),pack('A3',$hrf),pack('A255',$hmxnme),pack('L!',$nc2),pack('A10000',$hfld),pack('d',$xtemp[1]),pack('L!',$ierr),pack('A255',$herr));	#required to make DLL function call
		$SETMIXdll->Call($hmxnme, $hfmix, $hrf, $nc2, $hfld, $xtemp[1], $ierr, $herr, pack('L!',255), pack('L!',255), pack('L!',3), pack('L!',10000), pack('L!',255));		#(String, String, String, Long, String, Double, Long, String, Long, Long, Long, Long, Long) 
		($hmxnme, $hfmix, $hrf, $nc2, $hfld, $xtemp[1], $ierr, $herr) = (unpack('A255',$hfmix),unpack('A3',$hrf),unpack('A255',$hmxnme),unpack('L!',$nc2),unpack('A10000',$hfld),unpack('d',$xtemp[1]),unpack('L!',$ierr),unpack('A255',$herr));	#required to obtain DLL changes
		
	}elsif(($FluidName =~ m/,/) || ($FluidName =~ m/;/) || (($FluidName =~ m/\//) && ($FluidName =~ m/\(/))){
		$FluidNme = &Trim_Then_Append($FluidName);
		if(uc(substr($FluidNme,-4))eq "MASS"){$Mass = 1; $FluidNme = &Trim_Then_Append(substr($FluidNme,0,-4));}
		$nc2 = 0;
		if(($FluidNme =~ m/,/) || ($FluidNme =~ m/;/)){
			$FluidNme =~ s/;/,/;
			$FluidNme =~ s/,/;/;
			$FluidNme = substr($FluidNme,1) if (index($FluidNme,";") == 0);
			$FluidNme = $FluidNme . ";" unless (substr($FluidNme,-1) eq ";");
			@split_string = split(/;/,$FluidNme);
			foreach $split_element (0..$#split_string){
				$split_string[$split_element] = &Trim_Then_Append($split_string[$split_element]);
				if($split_element % 2){
					$nc2 += 1;
					$xtemp[$nc2] = $split_string[$split_element];
				}else{
					&CheckName(\$split_string[$split_element]);
					$split_string[$split_element] = $split_string[$split_element] . ".fld" unless($split_string[$split_element] =~ m/\.(fld)/);
					$a = $a . $FluidsDirectory . $split_string[$split_element] . "|";
				}
			}
			if($nc2 > $MaxComps){
				$ierr = 1;
				$herr = &Trim_Then_Append("Too many components","#");
				return $FluidName;;
			}
		}elsif((($FluidNme =~ m/\//) && ($FluidNme =~ m/\(/))){
			$FluidNme =~ s/\)//;
			$FluidNme = $FluidNme . "/" unless (substr($FluidNme,-1) eq "/");
			@xtemp = split(/\//,substr($FluidNme, index($FluidNme,"(") + 1));
			foreach $split_element (0..$#xtemp){
				$xtemp[$split_element] = &Trim_Then_Append($xtemp[$split_element]);
				$nc2 += 1;
			}
			unshift(@xtemp, undef);
			@split_string = split(/\//,substr($FluidNme, 0, index($FluidNme,"(")));
			foreach $split_element (0..$#split_string){
				$split_string[$split_element] = &Trim_Then_Append($split_string[$split_element]);
				&CheckName(\$split_string[$split_element]);
				$split_string[$split_element] = $split_string[$split_element] . ".fld" unless($split_string[$split_element] =~ m/\.(fld)/);
				$a = $a . $FluidsDirectory . $split_string[$split_element] . "|";
			}
			if(($nc2 > $MaxComps) || (scalar(@split_string) > $MaxComps)){
				$ierr = 1;
				$herr = &Trim_Then_Append("Too many components","#");
				return $FluidName;;
			}
		}
		$sum = 0;
		foreach $i (1..$#xtemp){$sum += $xtemp[$i];}
		if($sum <= 0){
			$ierr = 1;
			$herr = &Trim_Then_Append("Composition not set","#");
			return $FluidName;;
		}
		foreach $i (1..$#xtemp){$xtemp[$i] = $xtemp[$i] / $sum;}
		$hfld = $a;
		if($nc2 < 1){
			$ierr = 1;
			$herr = &Trim_Then_Append("Setup failed","#");
			return $FluidName;;
		}
		if($hfld ne $hfldOld){
		#To load the GERG-2004 pure fluid equations of state rather than the defaults
		#that come with Refprop, call the GERG04dll routine with a 1 as the second input.
		#($nc2, $ierr, $herr) = (pack('L!',$nc2),pack('L!',$ierr),pack('A255',$herr));
		# $GERG04dll->Call($nc2, pack('L!',1), $ierr, $herr, pack('L!',255));
		#($nc2, $ierr, $herr) = (unpack('L!',$nc2),unpack('L!',$ierr),unpack('A255',$herr));
		($nc2, $hfld, $hfmix, $hrf, $ierr, $herr) = (pack('L!',$nc2),pack('A10000',$hfld),pack('A255',$hfmix),pack('A3',$hrf),pack('L!',$ierr),pack('A255',$herr));	#required to make DLL function call
		$SETUPdll->Call($nc2, $hfld, $hfmix, $hrf, $ierr, $herr, pack('L!',10000), pack('L!',255), pack('L!',3), pack('L!',255));
		($nc2, $hfld, $hfmix, $hrf, $ierr, $herr) = (unpack('L!',$nc2),unpack('A10000',$hfld),unpack('A255',$hfmix),unpack('A3',$hrf),unpack('L!',$ierr),unpack('A255',$herr));
		$hfld = "" if ($ierr > 0);
		}
	}else{
		$nc2 = 1;		
		$FluidName = $FluidName . ".fld" unless (($FluidName =~ m/\.(fld)/) || ($FluidName =~ m/\.(ppf)/));
		if($FluidName =~ m/\\/){
			$hfld = $FluidName;
		}else{
			$hfld = $FluidsDirectory . $FluidName;
		}
		#.....Use call to SETMOD to change the equation of state for any of the
		#.....pure components from the default (recommended) values.
		#.....This should only be implemented by an experienced user.
		#if(($hfld =~ m/(argon)/) && ($nc2 == 1)){
		#	$hcomp = "FE1"; $htype = "EOS"; $hmix = $hcomp;
		#	($nc2, $htype, $hmix, $hcomp, $ierr, $herr)=(pack('L!',$nc2),pack('A3',$htype),pack('A255',$hfmix),pack('A60',$hcomp),pack('L!',$ierr),pack('A255',$herr));
		#	$SETMODdll->Call($nc2, $htype, $hmix, $hcomp, $ierr, $herr,pack('L!',3),pack('L!',3),pack('L!',60),pack('L!',255));
		#	($nc2, $htype, $hmix, $hcomp, $ierr, $herr)=(unpack('L!',$nc2),unpack('A3',$htype),unpack('A255',$hfmix),unpack('A60',$hcomp),unpack('L!',$ierr),unpack('A255',$herr));
		#}
		($nc2, $hfld, $hfmix, $hrf, $ierr, $herr) = (pack('L!',$nc2),pack('A10000',$hfld),pack('A255',$hfmix),pack('A3',$hrf),pack('L!',$ierr),pack('A255',$herr));	#required to make DLL function call
		$SETUPdll->Call($nc2, $hfld, $hfmix, $hrf, $ierr, $herr, pack('L!',10000), pack('L!',255), pack('L!',3), pack('L!',255));
		($nc2, $hfld, $hfmix, $hrf, $ierr, $herr) = (unpack('L!',$nc2),unpack('A10000',$hfld),unpack('A255',$hfmix),unpack('A3',$hrf),unpack('L!',$ierr),unpack('A255',$herr));
		$hfld = "" if ($ierr > 0);
	}
	$hfldOld = $hfld;
	#Use call to PREOSdll to change the equation of state to Peng Robinson for all calculations.
	#To revert back to the normal REFPROP EOS and models, use:  $PREOSdll->Call(pack('L!',0));
	#$PREOSdll->Call(pack('L!',2));
	if($Mass){
		@xkg = @xtemp;
		($xkg[1],$xtemp[1], $wmix) = (pack('d',$xkg[1]),pack('d',$xtemp[1]),pack('d',$wmix));
		$XMOLEdll->Call($xkg[1],$xtemp[1],$wmix);
		($xkg[1],$xtemp[1], $wmix) = (unpack('d',$xkg[1]),unpack('d',$xtemp[1]),unpack('d',$wmix));
	}
	if($ierr <= 0){
		$nc = $nc2;
		@x = @xtemp;
		$return_value = $FluidName;
		$FldOld = $FlNme;
		#Use the following line to calculate enthalpies and entropies on a reference state
		#based on the currently defined mixture, or to change to some other reference state.
		#The routine does not have to be called, but doing so will cause calculations
		#to be the same as those produced from the graphical interface for mixtures.
		($hrf, $x[1],$hRef,$sRef,$Tref,$pref,$ierr2,$herr2) = (pack('A3',$hrf),pack('d',$x[1]),pack('d',$hRef),pack('d',$sRef),pack('d',$Tref),pack('d',$pref),pack('L!',$ierr2),pack('A255',$herr2));
		$SETREFdll->Call($hrf, pack('L!',2), $x[1],$hRef,$sRef,$Tref,$pref,$ierr2,$herr2, pack('L!',3), pack('L!',255));
		($hrf, $x[1],$hRef,$sRef,$Tref,$pref,$ierr2,$herr2) = (unpack('A3',$hrf),unpack('d',$x[1]),unpack('d',$hRef),unpack('d',$sRef),unpack('d',$Tref),unpack('d',$pref),unpack('L!',$ierr2),unpack('A255',$herr2));
	}else{
		$return_value = &Trim_Then_Append($herr,"#");
		$FldOld = "";
	}
	return $return_value;
}

sub SetupUnits{	#(i)
	#Warning:  If any of these are changed (to make them the default) then the .DLL needs to be reloaded
	
	#Refprop Units
	$tUnits2 = "K";
	$taUnits2 = "K";
	$pUnits2 = "kPa";
	$dUnits2 = "mol/dm^3";
	$vUnits2 = "dm^3/mol";
	$hUnits2 = "J/mol";
	$sUnits2 = "J/mol-K";
	$wUnits2 = "m/s";
	$visUnits2 = "uPa-s";
	$tcxUnits2 = "W/m-K";
	$stUnits2 = "N/m";
	
	#Default units: (SI)
	$tUnits[0] = "K";
	$taUnits[0] = "K";
	$pUnits[0] = "MPa";
	$dUnits[0] = "kg/m^3";
	$vUnits[0] = "m^3/kg";
	$hUnits[0] = "kJ/kg";
	$sUnits[0] = "kJ/kg-K";
	$wUnits[0] = "m/s";
	$visUnits[0] = "uPa-s";
	$tcxUnits[0] = "mW/m-K";
	$stUnits[0] = "mN/m";
	
	#Default units but with K switched to C (SI with C)
	$tUnits[5] = "C";
	$taUnits[5] = "K";
	$pUnits[5] = "MPa";
	$dUnits[5] = "kg/m^3";
	$vUnits[5] = "m^3/kg";
	$hUnits[5] = "kJ/kg";
	$sUnits[5] = "kJ/kg-K";
	$wUnits[5] = "m/s";
	$visUnits[5] = "uPa-s";
	$tcxUnits[5] = "mW/m-K";
	$stUnits[5] = "mN/m";
	
	#Default units on a molar basis (Molar SI)
	$tUnits[6] = "K";
	$taUnits[6] = "K";
	$pUnits[6] = "MPa";
	$dUnits[6] = "mol/dm^3";
	$vUnits[6] = "dm^3/mol";
	$hUnits[6] = "J/mol";
	$sUnits[6] = "J/mol-K";
	$wUnits[6] = "m/s";
	$visUnits[6] = "uPa-s";
	$tcxUnits[6] = "mW/m-K";
	$stUnits[6] = "mN/m";
	
	#mks (mks)
	$tUnits[1] = "K";
	$taUnits[1] = "K";
	$pUnits[1] = "kPa";
	$dUnits[1] = "kg/m^3";
	$vUnits[1] = "m^3/kg";
	$hUnits[1] = "kJ/kg";
	$sUnits[1] = "kJ/kg-K";
	$wUnits[1] = "m/s";
	$visUnits[1] = "uPa-s";
	$tcxUnits[1] = "W/m-K";
	$stUnits[1] = "mN/m";
	
	#cgs (cgs)
	$tUnits[2] = "K";
	$taUnits[2] = "K";
	$pUnits[2] = "MPa";
	$dUnits[2] = "g/cm^3";
	$vUnits[2] = "cm^3/g";
	$hUnits[2] = "J/g";
	$sUnits[2] = "J/g-K";
	$wUnits[2] = "cm/s";
	$visUnits[2] = "uPa-s";
	$tcxUnits[2] = "mW/m-K";
	$stUnits[2] = "dyn/cm";
	
	#English (E)
	$tUnits[3] = "F";            #See comments above
	$taUnits[3] = "R";
	$pUnits[3] = "psia";
	$dUnits[3] = "lbm/ft^3";
	$vUnits[3] = "ft^3/lbm";
	$hUnits[3] = "Btu/lbm";
	$sUnits[3] = "Btu/lbm-R";
	$wUnits[3] = "ft/s";
	$visUnits[3] = "lbm/ft-s";
	$tcxUnits[3] = "Btu/h-ft-F";
	$stUnits[3] = "lbf/ft";
	
	#Molar English (molar E)
	$tUnits[7] = "F";
	$taUnits[7] = "R";
	$pUnits[7] = "psia";
	$dUnits[7] = "lbmol/ft^3";
	$vUnits[7] = "ft^3/lbmol";
	$hUnits[7] = "Btu/lbmol";
	$sUnits[7] = "Btu/lbmol-R";
	$wUnits[7] = "ft/s";
	$visUnits[7] = "lbm/ft-s";
	$tcxUnits[7] = "Btu/h-ft-F";
	$stUnits[7] = "lbf/ft";
	
	#Mixed (M)
	$tUnits[4] = "K";
	$taUnits[4] = "K";
	$pUnits[4] = "psia";
	$dUnits[4] = "g/cm^3";
	$vUnits[4] = "cm^3/g";
	$hUnits[4] = "J/g";
	$sUnits[4] = "J/g-K";
	$wUnits[4] = "m/s";
	$visUnits[4] = "uPa-s";
	$tcxUnits[4] = "mW/m-K";
	$stUnits[4] = "mN/m";
	
	#Mechanical Engineering units (MEUNITS)
	$tUnits[9] = "C";
	$taUnits[9] = "K";
	$pUnits[9] = "bar";
	$dUnits[9] = "g/cm^3";
	$vUnits[9] = "cm^3/g";
	$hUnits[9] = "J/g";
	$sUnits[9] = "J/g-K";
	$wUnits[9] = "cm/s";
	$visUnits[9] = "centipoise";
	$tcxUnits[9] = "mW/m-K";
	
	return 0;
}

sub Trim_Then_Append{
	my $text = shift;
	my $optional_prefix = shift || "";
	$text =~ s/^\s+//;	#removes all leading white spaces
	$text =~ s/\s+$//;	#removes all trailing white spaces
	$text = $optional_prefix . $text;
	return $text;
}

sub UnitConvert{	#(InputValue, UnitType As String, OldUnits As String, NewUnits As String)
	#InputValue is the value to be converted from OldUnits to NewUnits
	#UnitType is one of the following letters (one character only in most cases):
	#UnitType    Unit name                          SI units
	#  T         Temperature                            K
	#  P         Pressure                               Pa
	#  D         Density or specific volume         mol/m^3 or kg/m^3 (or m^3/mol or m^3/kg)
	#  H         Enthalpy or specific energy        J/mol or J/kg
	#  S         Entropy or heat capacity           J/mol-K or J/kg-K
	#  W         Speed of sound                         m/s
	#  U         Viscosity                              Pa-s
	#  K         Thermal conductivity                   W/m-K
	#  JT        Joule Thomson                          K/Pa
	#  L         Length                                 m
	#  A         Area                                   m^2
	#  V         Volume                                 m^3
	#  M         Mass                                   kg
	#  F         Force                                  N
	#  E         Energy                                 J
	#  Q         Power                                  W
	#  N         Surface tension                        N/m
	# Gage pressures can be used by adding "_g" to the unit, e.g., "MPa_g"
	# The different inputs for OldUnits and NewUnits can be found scattered in the text below.
	# Several examples are given below:
	# Temperature conversion, Kelvin to Fahrenheit(323.15,"T","K","F")
	# Pressure conversion, bar to mm of mercury head(1.01325,"P","bar","mmHg")
	# Density to specific density conversion, (1000.,"D","kg/m^3","cm^3/mol")
	
	my ($InputValue, $UnitType, $OldUnits, $NewUnits) = @_;
	my ($Value,$MolWt);	#VBA As Double
	my ($Tpe,$Unit1,$Unit2);	#VBA As String
	my ($Drct,$Gage,$Vacm);	#VBA As Integer
	our ($WMOLdll);
	
	return 0 if not looks_like_number($InputValue);
	return $InputValue if $NewUnits eq "";
	
	$Value = $InputValue;
	$Tpe = uc(&Trim_Then_Append($UnitType));
	$Unit1 = uc(&Trim_Then_Append($OldUnits));
	$Unit2 = uc(&Trim_Then_Append($NewUnits));
	
	($x[1],$wm) = (pack('d',$x[1]),pack('d',$wm));
	$WMOLdll->Call($x[1],$wm);
	($x[1],$wm) = (unpack('d',$x[1]),unpack('d',$wm));
	if($CompFlag == 1){
		($xliq[1],$wm) = (pack('d',$xliq[1]),pack('d',$wm));
		$WMOLdll->Call($xliq[1],$wm);
		($xliq[1],$wm) = (unpack('d',$xliq[1]),unpack('d',$wm));
	}
	if($CompFlag == 2){
		($xvap[1],$wm) = (pack('d',$xvap[1]),pack('d',$wm));
		$WMOLdll->Call($xliq[1],$wm);
		($xvap[1],$wm) = (unpack('d',$xvap[1]),unpack('d',$wm));
	}
	$MolWt = $wm;

	for($Drct = 1;$Drct >= -2; $Drct-=2){	#switch for converting to and from base unit type (+1 converts to -1 converts from)
		if($Tpe eq "T"){
			#-----------------------------------------------------------------------
			#   Temperature Conversion
			#-----------------------------------------------------------------------
			if($Unit1 eq "K"){	
				$Value = $Value;	#base unit, already Kelvin
			}elsif($Unit1 eq "C"){
				$Value = $Value + $Drct * $CtoK;	#Celsius to/from Kelvin
			}elsif($Unit1 eq "R"){
				$Value = $Value * $RtoK ** $Drct;	#Rankine to/from Kelvin
			}elsif($Unit1 eq "F"){
				if($Drct == 1){
					$Value = ($Value - 32) * $RtoK + $CtoK;	#Fahrenheit to Kelvin
				}else{
					$Value = ($Value - $CtoK) / $RtoK + 32;	#Kelvin to Fahrenheit
				}
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
		}elsif($Tpe eq "P"){
			#-----------------------------------------------------------------------
			#   Pressure Conversion
			#-----------------------------------------------------------------------
			if( ($Unit1 =~ m/(GAGE)/) || ($Unit1 =~ m/(_G)/) ){	#adjust for gauge pressure
				$Gage = 1;
				$Unit1 =~ s/(GAGE)//;
				$Unit1 =~ s/(_G)//;
				$Unit1 = &Trim_Then_Append($Unit1);
				$Value = $Value - $ATMtoMPa if $Drct == -1;
			}			
			if( ($Unit1 =~ m/(VACM)/) || ($Unit1 =~ m/(_V)/) ){	#adjust for vacuum
				$Vacm = 1;
				$Unit1 =~ s/(VACM)//;
				$Unit1 =~ s/(_V)//;
				$Unit1 = &Trim_Then_Append($Unit1);
				$Value = $ATMtoMPa - $Value if $Drct == -1;
			}
			if($Unit1 eq "PA"){
				$Value = $Value / 1000000 ** $Drct;	#Pa to/from MPa
			}elsif($Unit1 eq "KPA"){
				$Value = $Value / 1000 ** $Drct;	#kPa to/from MPa
			}elsif($Unit1 eq "MPA"){
				$Value = $Value;	#base unit, already MPa
			}elsif($Unit1 eq "GPA"){
				$Value = $Value * 1000 ** $Drct;	#GPa to/from MPa
			}elsif($Unit1 eq "BAR"){
				$Value = $Value * $BARtoMPA ** $Drct;	#bar to/from MPa
			}elsif($Unit1 eq "KBAR"){
				$Value = $Value * ($BARtoMPA * 1000) ** $Drct;	#kbar to/from MPa
			}elsif($Unit1 eq "ATM"){
				$Value = $Value * $ATMtoMPa ** $Drct;	#standard ATMosphere to/from MPa
			}elsif(($Unit1 eq "KGF/CM^2") || ($Unit1 eq "KG/CM^2") || ($Unit1 eq "ATA") || ($Unit1 eq "AT") || ($Unit1 eq "ATMA")){
				$Value = $Value * ($KGFtoN / 100) ** $Drct;	#kg force per sq cm to /from MPA
			}elsif(($Unit1 eq "PSI") || ($Unit1 eq "PSIA")){
				$Value = $Value * $PSIAtoMPA ** $Drct;		#lbs per sq inch to/from MPa
			}elsif($Unit1 eq "PSF"){
				$Value = $Value * ($PSIAtoMPA / 144) ** $Drct;	#lbs per sq ft to/from MPa
			}elsif(($Unit1 eq "MMHG") || ($Unit1 eq "TORR")){
				$Value = $Value * $MMHGtoMPA ** $Drct;	#mm of mercury head / torr to/from MPa
			}elsif($Unit1 eq "CMHG"){
				$Value = $Value * ($MMHGtoMPA * 10) ** $Drct;	#cm of mercury head to/from MPa
			}elsif($Unit1 eq "INHG"){
				$Value = $Value * ($MMHGtoMPA * $INtoM * 1000) ** $Drct;	#inch of mercury head to/from MPa
			}elsif($Unit1 eq "INH2O"){
				$Value = $Value * $INH2OtoMPA ** $Drct;	#inch of water gauge to/from MPa
			}elsif($Unit1 eq "PSIG"){	
				if($Drct == 1){
					$Value = $PSIAtoMPA * $Value + $ATMtoMPa;	#psi gauge to MPa
				}else{
					$Value = ($Value - $ATMtoMPa) / $PSIAtoMPA;	#MPa to psi gauge
				}
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
			$Value = $Value + $ATMtoMPa if(($Gage == 1) && ($Drct == 1)); 
			$Value = $ATMtoMPa - $Value if(($Vacm == 1) && ($Drct == 1)); 
		}elsif($Tpe eq "D"){
			#-----------------------------------------------------------------------
			#   Density Conversion
			#-----------------------------------------------------------------------
			$Value = 1e-50 if $Value == 0;
			if(($Unit1 eq "MOL/DM^3") || ($Unit1 eq "MOL/L") || ($Unit1 eq "KMOL/M^3")){
				$Value = $Value;	#base unit, already Mole per litre
			}elsif(($Unit1 eq "MOL/CM^3") || ($Unit1 eq "MOL/CC")){
				$Value = $Value * 1000 ** $Drct;
			}elsif($Unit1 eq "MOL/M^3"){
				$Value = $Value / 1000 ** $Drct;
			}elsif($Unit1 eq "KG/M^3"){
				$Value = $Value / $MolWt ** $Drct;
			}elsif(($Unit1 eq "KG/DM^3") || ($Unit1 eq "KG/L")){
				$Value = $Value * (1000 / $MolWt) ** $Drct;
			}elsif(($Unit1 eq "G/DM^3") || ($Unit1 eq "G/L")){
				$Value = $Value * (1 / $MolWt) ** $Drct;
			}elsif(($Unit1 eq "G/CC") || ($Unit1 eq "G/CM^3") || ($Unit1 eq "G/ML")){
				$Value = $Value * (1000 / $MolWt) ** $Drct;
			}elsif($Unit1 eq "G/DM^3"){
				$Value = $Value * (1 / $MolWt) ** $Drct;
			}elsif(($Unit1 eq "LBM/FT^3") || ($Unit1 eq "LB/FT^3")){
				$Value = $Value * ($LBMtoKG / $FT3toM3 / $MolWt) ** $Drct;
			}elsif($Unit1 eq "LBMOL/FT^3"){
				$Value = $Value * ($LBMtoKG / $FT3toM3) ** $Drct;
			}elsif($Unit1 eq "SLUG/FT^3"){
				$Value = $Value * ($LBMtoKG / $FT3toM3 / $MolWt * $KGFtoN / $FTtoM) ** $Drct;
			}elsif($Unit1 eq "LBMOL/GAL"){
				$Value = $Value * ($LBMtoKG / $GALLONtoM3) ** $Drct;
			}elsif(($Unit1 eq "LB/GAL") || ($Unit1 eq "LBM/GAL")){
				$Value = $Value * ($LBMtoKG / $GALLONtoM3 / $MolWt) ** $Drct;
			#----------------------------------------------------------------------
			#  Specific Volume Conversion
			#----------------------------------------------------------------------
			}elsif(($Unit1 eq "DM^3/MOL") || ($Unit1 eq "L/MOL") || ($Unit1 eq "M^3/KMOL")){
				$Value = 1 / $Value;	#base unit, Litre per mole
			}elsif(($Unit1 eq "CM^3/MOL") || ($Unit1 eq "CC/MOL") || ($Unit1 eq "ML/MOL")){
				$Value = 1000 / $Value;
			}elsif($Unit1 eq "M^3/MOL"){
				$Value = 1 / $Value / 1000;
			}elsif($Unit1 eq "M^3/KG"){
				$Value = 1 / $Value / $MolWt;
			}elsif(($Unit1 eq "DM^3/KG") || ($Unit1 eq "L/KG")){
				$Value = 1000 / $Value / $MolWt;
			}elsif(($Unit1 eq "CC/G") || ($Unit1 eq "CM^3/G") || ($Unit1 eq "ML/G")){
				$Value = 1000 / $Value / $MolWt;
			}elsif($Unit1 eq "DM^3/G"){
				$Value = 1 / $Value / $MolWt;
			}elsif(($Unit1 eq "FT^3/LBM") || ($Unit1 eq "FT^3/LB")){
				$Value = 1 / $Value * ($LBMtoKG / $FT3toM3 / $MolWt);
			}elsif($Unit1 eq "FT^3/LBMOL"){
				$Value = 1 / $Value * ($LBMtoKG / $FT3toM3);
			}elsif($Unit1 eq "FT^3/SLUG"){
				$Value = 1 / $Value * ($LBMtoKG / $FT3toM3 / $MolWt * $KGFtoN / $FTtoM);
			}elsif($Unit1 eq "GAL/LBMOL"){
				$Value = 1 / $Value * ($LBMtoKG / $GALLONtoM3);
			}elsif(($Unit1 eq "GAL/LB") || ($Unit1 eq "GAL/LBM")){
				$Value = 1 / $Value * ($LBMtoKG / $GALLONtoM3 / $MolWt);
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
			$Value = 0 if(abs($Value) < 1e-30);
		}elsif($Tpe eq "H"){
			#-----------------------------------------------------------------------
			#   Specific Energy and Enthalpy Conversions
			#-----------------------------------------------------------------------
			if(($Unit1 eq "J/MOL") || ($Unit1 eq "KJ/KMOL")){
				$Value = $Value;	#base unit, already Joule per mole
			}elsif($Unit1 eq "KJ/MOL"){
				$Value = $Value * 1000 ** $Drct;
			}elsif($Unit1 eq "MJ/MOL"){
				$Value = $Value * 1000000 ** $Drct;
			}elsif(($Unit1 eq "KJ/KG") || ($Unit1 eq "J/G")){
				$Value = $MolWt ** $Drct * $Value;
			}elsif($Unit1 eq "J/KG"){
				$Value = ($MolWt / 1000) ** $Drct * $Value;
			}elsif($Unit1 eq "M^2/S^2"){
				$Value = ($MolWt / 1000) ** $Drct * $Value;
			}elsif($Unit1 eq "FT^2/S^2"){
				$Value = ($MolWt / 1000 * $FTtoM ** 2) ** $Drct * $Value;
			}elsif(($Unit1 eq "CAL/MOL") || ($Unit1 eq "KCAL/KMOL")){
				$Value = $CALtoJ ** $Drct * $Value;
			}elsif(($Unit1 eq "CAL/G") || ($Unit1 eq "KCAL/KG")){
				$Value = ($CALtoJ * $MolWt) ** $Drct * $Value;
			}elsif(($Unit1 eq "BTU/LBM") || ($Unit1 eq "BTU/LB")){
				$Value = ($BTUtoKJ / $LBMtoKG * $MolWt) ** $Drct * $Value;
			}elsif($Unit1 eq "BTU/LBMOL"){
				$Value = ($BTUtoKJ / $LBMtoKG) ** $Drct * $Value;
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
		}elsif($Tpe eq "S"){
			#-----------------------------------------------------------------------
			#   Entropy and Heat Capacity Conversions
			#-----------------------------------------------------------------------
			if(($Unit1 eq "J/MOL-K") || ($Unit1 eq "KJ/KMOL-K")){
				$Value = $Value;	#base unit, already Joule per mole Kelvin
			}elsif($Unit1 eq "KJ/MOL-K"){
				$Value = $Value * 1000 ** $Drct;
			}elsif(($Unit1 eq "KJ/KG-K") || ($Unit1 eq "J/G-K")){
				$Value = $MolWt ** $Drct * $Value;
			}elsif($Unit1 eq "J/KG-K"){
				$Value = ($MolWt / 1000) ** $Drct * $Value;
			}elsif(($Unit1 eq "BTU/LBM-R") || ($Unit1 eq "BTU/LB-R")){
				$Value = ($BTUtoKJ / $LBMtoKG / $RtoK * $MolWt) ** $Drct * $Value;
			}elsif($Unit1 eq "BTU/LBMOL-R"){
				$Value = ($BTUtoKJ / $LBMtoKG / $RtoK) ** $Drct * $Value;
			}elsif(($Unit1 eq "CAL/G-K") || ($Unit1 eq "CAL/G-C") || ($Unit1 eq "KCAL/KG-K") || ($Unit1 eq "KCAL/KG-C")){
				$Value = ($CALtoJ * $MolWt) ** $Drct * $Value;
			}elsif(($Unit1 eq "CAL/MOL-K") || ($Unit1 eq "CAL/MOL-C")){
				$Value = $CALtoJ ** $Drct * $Value;
			}elsif($Unit1 eq "FT-LBF/LBMOL-R"){
				$Value = ($FTLBFtoJ / $LBMtoKG / $RtoK / 1000) ** $Drct * $Value;
			}elsif($Unit1 eq "CP/R"){
				$Value = $Rgas ** $Drct * $Value * 1000;
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
		}elsif($Tpe eq "W"){
			#-----------------------------------------------------------------------
			#   Speed of Sound Conversion
			#-----------------------------------------------------------------------
			if($Unit1 eq "M/S"){
				$Value = $Value;	#base unit, already metres per sec
			}elsif($Unit1 eq "M^2/S^2"){
				$Value = sqrt($Value);
			}elsif($Unit1 eq "CM/S"){
				$Value = $Value / 100 ** $Drct;
			}elsif($Unit1 eq "KM/H"){
				$Value = $Value * (1000 / $HtoS) ** $Drct;
			}elsif($Unit1 eq "FT/S"){
				$Value = $Value * $FTtoM ** $Drct;
			}elsif($Unit1 eq "IN/S"){
				$Value = $Value * $INtoM ** $Drct;
			}elsif(($Unit1 eq "MILE/H") || ($Unit1 eq "MPH")){
				$Value = $Value * ($INtoM * 63360 / $HtoS) ** $Drct;
			}elsif($Unit1 eq "KNOT"){
				$Value = $Value * 0.5144444444 ** $Drct;
			}elsif($Unit1 eq "MACH"){
				$Value = $Value * sqrt(1.4 * 298.15 * 8314.51 / 28.95853816) ** $Drct;
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
		}elsif($Tpe eq "U"){
			#-----------------------------------------------------------------------
			#   Viscosity Conversion
			#-----------------------------------------------------------------------
			if(($Unit1 eq "PA-S") || ($Unit1 eq "KG/M-S")){
				$Value = $Value;	#base unit, already Pascal seconds
			}elsif($Unit1 eq "MPA-S"){	#Note:  This is milliPa-s, not MPa-s
			  $Value = $Value / 1000 ** $Drct;
			}elsif($Unit1 eq "UPA-S"){
			  $Value = $Value / 1000000 ** $Drct;
			}elsif(($Unit1 eq "G/CM-S") || ($Unit1 eq "POISE")){
			  $Value = $Value / 10 ** $Drct;
			}elsif($Unit1 eq "CENTIPOISE"){
			  $Value = $Value / 1000 ** $Drct;
			}elsif(($Unit1 eq "MILLIPOISE") || ($Unit1 eq "MPOISE")){
			  $Value = $Value / 10000 ** $Drct;
			}elsif(($Unit1 eq "MICROPOISE") || ($Unit1 eq "UPOISE")){
			  $Value = $Value / 10000000 ** $Drct;
			}elsif(($Unit1 eq "LBM/FT-S") || ($Unit1 eq "LB/FT-S")){
			  $Value = $Value * ($LBMtoKG / $FTtoM) ** $Drct;
			}elsif($Unit1 eq "LBF-S/FT^2"){
			  $Value = $Value * ($LBFtoN / $FTtoM ** 2) ** $Drct;
			}elsif(($Unit1 eq "LBM/FT-H") || ($Unit1 eq "LB/FT-H")){
			  $Value = $Value * ($LBMtoKG / $FTtoM / $HtoS) ** $Drct;
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
		}elsif($Tpe eq "K"){
			#-----------------------------------------------------------------------
			#   Thermal Conductivity Conversion
			#-----------------------------------------------------------------------
			if($Unit1 eq "MW/M-K"){
				$Value = $Value;	#base unit, already Mega watt per metre Kelvin
			}elsif($Unit1 eq "W/M-K"){
				$Value = $Value * 1000 ** $Drct;
			}elsif($Unit1 eq "G-CM/S^3-K"){
				$Value = $Value / 100 ** $Drct;
			}elsif($Unit1 eq "KG-M/S^3-K"){
				$Value = $Value * 1000 ** $Drct;
			}elsif($Unit1 eq "CAL/S-CM-K"){
				$Value = $Value * ($CALtoJ * 100000) ** $Drct;
			}elsif($Unit1 eq "KCAL/HR-M-K"){
				$Value = $Value * ($CALtoJ * 100000 * 1000 / 100 / 3600) ** $Drct;
			}elsif(($Unit1 eq "LBM-FT/S^3-F") || ($Unit1 eq "LB-FT/S^3-F")){
				$Value = $Value * (1000 * $LBMtoKG * $FTtoM / $RtoK) ** $Drct;
			}elsif($Unit1 eq "LBF/S-F"){
				$Value = $Value * (1000 * $LBFtoN / $RtoK) ** $Drct;
			}elsif($Unit1 eq "BTU/H-FT-F"){
				$Value = $Value * (1000 * $BTUtoW / $HtoS / $FTtoM / $RtoK) ** $Drct;
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
		}elsif($Tpe eq "JT"){
			#-----------------------------------------------------------------------
			#   Joule-Thomson Conversion
			#-----------------------------------------------------------------------
			if(($Unit1 eq "K/MPA") || ($Unit1 eq "C/MPA")){
				$Value = $Value;	#base unit, already Kelvin (Celsius) per MPa
			}elsif(($Unit1 eq "K/KPA") || ($Unit1 eq "C/KPA")){
				$Value = $Value * 1000 ** $Drct;
			}elsif(($Unit1 eq "K/PA") || ($Unit1 eq "C/PA")){
				$Value = $Value * 1000000 ** $Drct;
			}elsif($Unit1 eq "C/ATM"){
				$Value = $Value / $ATMtoMPa ** $Drct;
			}elsif($Unit1 eq "C/BAR"){
			  $Value = $Value / $BARtoMPA ** $Drct;
			}elsif(($Unit1 eq "K/PSI") || ($Unit1 eq "K/PSIA")){
			  $Value = $Value / $PSIAtoMPA ** $Drct;
			}elsif(($Unit1 eq "F/PSI") || ($Unit1 eq "F/PSIA") || ($Unit1 eq "R/PSIA")){
			  $Value = $Value / ($PSIAtoMPA / $RtoK) ** $Drct;
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
		}elsif($Tpe eq "L"){
			#-----------------------------------------------------------------------
			#   Length Conversion
			#-----------------------------------------------------------------------
			if(($Unit1 eq "METER") || ($Unit1 eq "M")){
				$Value = $Value;	#base unit, already Metres
			}elsif($Unit1 eq "DM"){
				$Value = $Value / 10 ** $Drct;
			}elsif($Unit1 eq "CM"){
				$Value = $Value / 100 ** $Drct;
			}elsif($Unit1 eq "MM"){
				$Value = $Value / 1000 ** $Drct;
			}elsif($Unit1 eq "KM"){
				$Value = $Value * 1000 ** $Drct;
			}elsif(($Unit1 eq "INCH") || ($Unit1 eq "IN")){
				$Value = $Value * $INtoM ** $Drct;
			}elsif(($Unit1 eq "FOOT") || ($Unit1 eq "FT")){
				$Value = $Value * $FTtoM ** $Drct;
			}elsif(($Unit1 eq "YARD") || ($Unit1 eq "YD")){
				$Value = $Value * ($INtoM * 36) ** $Drct;
			}elsif(($Unit1 eq "MILE") || ($Unit1 eq "MI")){
				$Value = $Value * ($INtoM * 63360) ** $Drct;
			}elsif($Unit1 eq "LIGHT YEAR"){
				$Value = $Value * 9.46055E+15 ** $Drct;
			}elsif($Unit1 eq "ANGSTROM"){
				$Value = $Value / 10000000000 ** $Drct;
			}elsif($Unit1 eq "FATHOM"){
				$Value = $Value * ($FTtoM * 6) ** $Drct;
			}elsif($Unit1 eq "MIL"){
				$Value = $Value * ($INtoM / 1000) ** $Drct;
			}elsif($Unit1 eq "ROD"){
				$Value = $Value * ($INtoM * 16.5 * 12) ** $Drct;
			}elsif($Unit1 eq "PARSEC"){
				$Value = $Value * (30837400000000 * 1000) ** $Drct;
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
		}elsif($Tpe eq "A"){
		#-----------------------------------------------------------------------
		#   Area Conversion
		#-----------------------------------------------------------------------
			if(($Unit1 eq "METER^2") || ($Unit1 = "M^2")){
				$Value = $Value;	#base unit, already Sq Metres
			}elsif($Unit1 eq "CM^2"){
				$Value = $Value / 10000 ** $Drct;
			}elsif($Unit1 eq "MM^2"){
				$Value = $Value / 1000000 ** $Drct;
			}elsif($Unit1 eq "KM^2"){
				$Value = $Value * 1000000 ** $Drct;
			}elsif(($Unit1 eq "INCH^2") || ($Unit1 eq "IN^2")){
				$Value = $Value * ($INtoM ** 2) ** $Drct;
			}elsif(($Unit1 eq "FOOT^2") || ($Unit1 eq "FT^2")){
				$Value = $Value * ($FTtoM ** 2) ** $Drct;
			}elsif(($Unit1 eq "YARD^2") || ($Unit1 eq "YD^2")){
				$Value = $Value * (($INtoM * 36) ** 2) ** $Drct;
			}elsif(($Unit1 eq "MILE^2") || ($Unit1 eq "MI^2")){
				$Value = $Value * (($INtoM * 63360) ** 2) ** $Drct;
			}elsif($Unit1 eq "ACRE"){
				$Value = $Value * (($INtoM * 36) ** 2 * 4840) ** $Drct;
			}elsif($Unit1 eq "BARN"){
				$Value = $Value * 1E-28 ** $Drct;
			}elsif($Unit1 eq "HECTARE"){
				$Value = $Value * 10000 ** $Drct;
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
		}elsif($Tpe eq "V"){
			#-----------------------------------------------------------------------
			#   Volume Conversion (Note: not specific volume)
			#-----------------------------------------------------------------------
			if(($Unit1 eq "METER^3") || ($Unit1 eq "M^3")){
				$Value = $Value;	#base unit, already Cubic Metres
			}elsif($Unit1 eq "CM^3"){
				$Value = $Value / 1000000 ** $Drct;
			}elsif(($Unit1 eq "LITER") || ($Unit1 eq "L") || ($Unit1 eq "DM^3")){
				$Value = $Value / 1000 ** $Drct;
			}elsif(($Unit1 eq "INCH^3") || ($Unit1 eq "IN^3")){
				$Value = $Value * $IN3toM3 ** $Drct;
			}elsif(($Unit1 eq "FOOT^3") || ($Unit1 eq "FT^3")){
				$Value = $Value * ($IN3toM3 * 12 ** 3) ** $Drct;
			}elsif(($Unit1 eq "YARD^3") || ($Unit1 eq "YD^3")){
				$Value = $Value * ($IN3toM3 * 36 ** 3) ** $Drct;
			}elsif(($Unit1 eq "GALLON") || ($Unit1 eq "GAL")){
				$Value = $Value * $GALLONtoM3 ** $Drct;
			}elsif(($Unit1 eq "QUART") || ($Unit1 eq "QT")){
				$Value = $Value * ($GALLONtoM3 / 4) ** $Drct;
			}elsif(($Unit1 eq "PINT") || ($Unit1 eq "PT")){
				$Value = $Value * ($GALLONtoM3 / 8) ** $Drct;
			}elsif($Unit1 eq "CUP"){
				$Value = $Value * ($GALLONtoM3 / 16) ** $Drct;
			}elsif($Unit1 eq "OUNCE"){
				$Value = $Value * ($GALLONtoM3 / 128) ** $Drct;
			}elsif(($Unit1 eq "TABLESPOON") || ($Unit1 eq "TBSP")){
				$Value = $Value * ($GALLONtoM3 / 256) ** $Drct;
			}elsif(($Unit1 eq "TEASPOON") || ($Unit1 eq "TSP")){
				$Value = $Value * ($GALLONtoM3 / 768) ** $Drct;
			}elsif($Unit1 eq "C||D"){
				$Value = $Value * ($FT3toM3 * 128) ** $Drct;
			}elsif($Unit1 eq "BARREL"){
				$Value = $Value * ($GALLONtoM3 * 42) ** $Drct;
			}elsif($Unit1 eq "BOARD FOOT"){
				$Value = $Value * ($IN3toM3 * 144) ** $Drct;
			}elsif($Unit1 eq "BUSHEL"){
				$Value = $Value * 0.03523907016688 ** $Drct;
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
		}elsif($Tpe eq "M"){
			#-----------------------------------------------------------------------
			#   Mass Conversion
			#-----------------------------------------------------------------------
			if($Unit1 eq "KG"){
				$Value = $Value;	#base unit, already kilograms
			}elsif($Unit1 eq "G"){
				$Value = $Value / 1000 ** $Drct;
			}elsif($Unit1 eq "MG"){	#milligram
				$Value = $Value / 1000000 ** $Drct;
			}elsif(($Unit1 eq "LBM") || ($Unit1 eq "LB")){
				$Value = $Value * $LBMtoKG ** $Drct;
			}elsif($Unit1 eq "GRAIN"){
				$Value = $Value * ($LBMtoKG / 7000) ** $Drct;
			}elsif($Unit1 eq "SLUG"){
				$Value = $Value * ($KGFtoN * $LBMtoKG / $FTtoM) ** $Drct;
			}elsif($Unit1 eq "TON"){
				$Value = $Value * ($LBMtoKG * 2000) ** $Drct;
			}elsif($Unit1 eq "TONNE"){
				$Value = $Value * 1000 ** $Drct;
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
		}elsif($Tpe eq "F"){
			#-----------------------------------------------------------------------
			#   Force Conversion
			#-----------------------------------------------------------------------
			if(($Unit1 eq "NEWTON") || ($Unit1 eq "N")){
				$Value = $Value;	#base unit, already Newtons
			}elsif($Unit1 eq "MN"){ #milliNewtons
				$Value = $Value / 1000 ** $Drct;
			}elsif($Unit1 eq "KGF"){
				$Value = $Value * $KGFtoN ** $Drct;
			}elsif($Unit1 eq "DYNE"){
				$Value = $Value / 100000 ** $Drct;
			}elsif($Unit1 eq "LBF"){
				$Value = $Value * $LBFtoN ** $Drct;
			}elsif($Unit1 eq "POUNDAL"){
				$Value = $Value * ($LBMtoKG * $FTtoM) ** $Drct;
			}elsif($Unit1 eq "OZF"){
				$Value = $Value * ($LBFtoN / 16) ** $Drct;
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
		}elsif($Tpe eq "E"){
			#-----------------------------------------------------------------------
			#   Energy Conversion
			#-----------------------------------------------------------------------
			if(($Unit1 eq "JOULE") || ($Unit1 eq "J")){
				$Value = $Value;	#base unit, already Joules
			}elsif($Unit1 eq "KJ"){
				$Value = $Value * 1000 ** $Drct;
			}elsif($Unit1 eq "MJ"){
				$Value = $Value * 1000000 ** $Drct;
			}elsif($Unit1 eq "KW-H"){
				$Value = $Value * ($HtoS * 1000) ** $Drct;
			}elsif($Unit1 eq "CAL"){
				$Value = $CALtoJ ** $Drct * $Value;
			}elsif($Unit1 eq "KCAL"){
				$Value = $Value * ($CALtoJ * 1000) ** $Drct;
			}elsif($Unit1 eq "ERG"){
				$Value = $Value / 10000000 ** $Drct;
			}elsif($Unit1 eq "BTU"){
				$Value = $Value * ($BTUtoKJ * 1000) ** $Drct;
			}elsif($Unit1 eq "FT-LBF"){
				$Value = $Value * $FTLBFtoJ ** $Drct;
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
		}elsif($Tpe eq "Q"){
			#-----------------------------------------------------------------------
			#   Power Conversion
			#-----------------------------------------------------------------------
			if(($Unit1 eq "WATT") || ($Unit1 eq "W")){
				$Value = $Value;	#base unit, already Watts
			}elsif(($Unit1 eq "KWATT") || ($Unit1 eq "KW")){
				$Value = $Value * 1000 ** $Drct;
			}elsif($Unit1 eq "BTU/S"){
				$Value = $Value * $BTUtoW ** $Drct;
			}elsif($Unit1 eq "BTU/MIN"){
				$Value = $Value * ($BTUtoW / 60) ** $Drct;
			}elsif($Unit1 eq "BTU/H"){
				$Value = $Value * ($BTUtoW / $HtoS) ** $Drct;
			}elsif($Unit1 eq "CAL/S"){
				$Value = $Value * $CALtoJ ** $Drct;
			}elsif($Unit1 eq "KCAL/S"){
				$Value = $Value * ($CALtoJ * 1000) ** $Drct;
			}elsif($Unit1 eq "CAL/MIN"){
				$Value = $Value * ($CALtoJ / 60) ** $Drct;
			}elsif($Unit1 eq "KCAL/MIN"){
				$Value = $Value * ($CALtoJ / 60 * 1000) ** $Drct;
			}elsif($Unit1 eq "FT-LBF/S"){
				$Value = $Value * $FTLBFtoJ ** $Drct;
			}elsif($Unit1 eq "FT-LBF/MIN"){
				$Value = $Value * ($FTLBFtoJ / 60) ** $Drct;
			}elsif($Unit1 eq "FT-LBF/H"){
				$Value = $Value * ($FTLBFtoJ / $HtoS) ** $Drct;
			}elsif($Unit1 eq "HP"){
				$Value = $Value * $HPtoW ** $Drct;
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
		}elsif($Tpe eq "N"){
			#-----------------------------------------------------------------------
			#   Surface Tension Conversion
			#-----------------------------------------------------------------------
			if($Unit1 eq "N/M"){
				$Value = $Value;	#base unit, already Newton per metre
			}elsif($Unit1 eq "MN/M"){
				$Value = $Value / 1000 ** $Drct;
			}elsif(($Unit1 eq "DYNE/CM") || ($Unit1 eq "DYN/CM")){
				$Value = $Value / 1000 ** $Drct;
			}elsif($Unit1 eq "LBF/FT"){
				$Value = $Value * $LBFTtoNM ** $Drct;
			}else{
				return &Trim_Then_Append("Undefined input unit","#");
			}
		}
		$Unit1 = $Unit2;
	}
	return $Value;
}

1;